# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands.context import (
    MISSING,
    TYPE_CHECKING,
    Any,
    BotT,
    CogT,
    Context,
    Dict,
    Generic,
    List,
    Message,
    Optional,
    P,
    T,
    TypeVar,
    Union,
    annotations,
    inspect,
    re,
    versacord,
)

__all__ = "Context"



# SPDX-License-Identifier: MIT

from __future__ import annotations

from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    ClassVar,
    Dict,
    Generator,
    List,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from . import enums, utils
from .asset import Asset
from .auto_moderation import AutoModerationAction, AutoModerationTriggerMetadata
from .colour import Colour
from .invite import Invite
from .mixins import Hashable
from .object import Object
from .permissions import PermissionOverwrite, Permissions

__all__ = (
    "AuditLogDiff",
    "AuditLogChanges",
    "AuditLogEntry",
)


if TYPE_CHECKING:
    import datetime

    from . import abc
    from .auto_moderation import AutoModerationRule
    from .emoji import Emoji
    from .guild import Guild
    from .member import Member
    from .role import Role
    from .stage_instance import StageInstance
    from .sticker import GuildSticker
    from .threads import Thread
    from .types.audit_log import (
        AuditLogChange as AuditLogChangePayload,
        AuditLogEntry as AuditLogEntryPayload,
    )
    from .types.auto_moderation import (
        AutoModerationAction as AutoModerationActionPayload,
        AutoModerationTriggerMetadata as AutoModerationTriggerMetadataPayload,
    )
    from .types.channel import PermissionOverwrite as PermissionOverwritePayload
    from .types.role import Role as RolePayload
    from .types.snowflake import Snowflake
    from .user import User

    AuditTarget = Union[
        Guild,
        abc.GuildChannel,
        Member,
        User,
        Role,
        Invite,
        Emoji,
        StageInstance,
        GuildSticker,
        Thread,
        Object,
        AutoModerationRule,
        None,
    ]


def _transform_permissions(entry: AuditLogEntry, data: str) -> Permissions:
    return Permissions(int(data))


def _transform_color(entry: AuditLogEntry, data: int) -> Colour:
    return Colour(data)


def _transform_snowflake(entry: AuditLogEntry, data: Snowflake) -> int:
    return int(data)


def _transform_channel(
    entry: AuditLogEntry, data: Optional[Snowflake]
) -> Optional[Union[abc.GuildChannel, Object]]:
    if data is None:
        return None
    return entry.guild.get_channel(int(data)) or Object(id=data)


def _transform_member_id(
    entry: AuditLogEntry, data: Optional[Snowflake]
) -> Union[Member, User, None]:
    if data is None:
        return None
    return entry._get_member(int(data))


def _transform_guild_id(entry: AuditLogEntry, data: Optional[Snowflake]) -> Optional[Guild]:
    if data is None:
        return None
    return entry._state._get_guild(int(data))


def _transform_overwrites(
    entry: AuditLogEntry, data: List[PermissionOverwritePayload]
) -> List[Tuple[Object, PermissionOverwrite]]:
    overwrites = []
    for elem in data:
        allow = Permissions(int(elem["allow"]))
        deny = Permissions(int(elem["deny"]))
        ow = PermissionOverwrite.from_pair(allow, deny)

        ow_type = elem["type"]
        ow_id = int(elem["id"])
        target = None
        if str(ow_type) == "0":
            target = entry.guild.get_role(ow_id)
        elif str(ow_type) == "1":
            target = entry._get_member(ow_id)

        if target is None:
            target = Object(id=ow_id)

        overwrites.append((target, ow))

    return overwrites


def _transform_icon(entry: AuditLogEntry, data: Optional[str]) -> Optional[Asset]:
    if data is None:
        return None
    return Asset._from_guild_icon(entry._state, entry.guild.id, data)


def _transform_avatar(entry: AuditLogEntry, data: Optional[str]) -> Optional[Asset]:
    if data is None:
        return None
    return Asset._from_avatar(entry._state, entry._target_id, data)  # type: ignore


def _guild_hash_transformer(path: str) -> Callable[[AuditLogEntry, Optional[str]], Optional[Asset]]:
    def _transform(entry: AuditLogEntry, data: Optional[str]) -> Optional[Asset]:
        if data is None:
            return None
        return Asset._from_guild_image(entry._state, entry.guild.id, data, path=path)

    return _transform


T = TypeVar("T")
E = TypeVar("E", bound=enums.Enum)


def _enum_transformer(enum: Type[E]) -> Callable[[AuditLogEntry, int], E]:
    def _transform(entry: AuditLogEntry, data: int) -> E:
        return enums.try_enum(enum, data)

    return _transform


def _transform_type(entry: AuditLogEntry, data: int) -> Union[enums.ChannelType, enums.StickerType]:
    if entry.action.name.startswith("sticker_"):
        return enums.try_enum(enums.StickerType, data)
    else:
        return enums.try_enum(enums.ChannelType, data)


def _list_transformer(
    func: Callable[[AuditLogEntry, Any], T]
) -> Callable[[AuditLogEntry, Any], List[T]]:
    def _transform(entry: AuditLogEntry, data: Any) -> List[T]:
        if not data:
            return []
        return [func(entry, value) for value in data if value is not None]

    return _transform


def _transform_auto_moderation_action(
    entry: AuditLogEntry, data: Optional[AutoModerationActionPayload]
) -> Optional[AutoModerationAction]:
    if data is None:
        return None
    return AutoModerationAction.from_data(data)


def _transform_auto_moderation_trigger_metadata(
    entry: AuditLogEntry, data: Optional[AutoModerationTriggerMetadataPayload]
) -> Optional[AutoModerationTriggerMetadata]:
    if data is None:
        return None
    return AutoModerationTriggerMetadata.from_data(data)


def _transform_role(
    entry: AuditLogEntry, data: Optional[Snowflake]
) -> Optional[Union[Role, Object]]:
    if data is None:
        return None
    role = entry.guild.get_role(int(data))
    return role or Object(id=data)


class AuditLogDiff:
    def __len__(self) -> int:
        return len(self.__dict__)

    def __iter__(self) -> Generator[Tuple[str, Any], None, None]:
        yield from self.__dict__.items()

    def __repr__(self) -> str:
        values = " ".join("%s=%r" % item for item in self.__dict__.items())
        return f"<AuditLogDiff {values}>"

    if TYPE_CHECKING:

        def __getattr__(self, item: str) -> Any:
            ...

        def __setattr__(self, key: str, value: Any) -> Any:
            ...


Transformer = Callable[["AuditLogEntry", Any], Any]


class AuditLogChanges:
    # fmt: off
    TRANSFORMERS: ClassVar[Dict[str, Tuple[Optional[str], Optional[Transformer]]]] = {
        "verification_level":            (None, _enum_transformer(enums.VerificationLevel)),
        "explicit_content_filter":       (None, _enum_transformer(enums.ContentFilter)),
        "allow":                         (None, _transform_permissions),
        "deny":                          (None, _transform_permissions),
        "permissions":                   (None, _transform_permissions),
        "id":                            (None, _transform_snowflake),
        "color":                         ("colour", _transform_color),
        "owner_id":                      ("owner", _transform_member_id),
        "inviter_id":                    ("inviter", _transform_member_id),
        "channel_id":                    ("channel", _transform_channel),
        "afk_channel_id":                ("afk_channel", _transform_channel),
        "system_channel_id":             ("system_channel", _transform_channel),
        "widget_channel_id":             ("widget_channel", _transform_channel),
        "rules_channel_id":              ("rules_channel", _transform_channel),
        "public_updates_channel_id":     ("public_updates_channel", _transform_channel),
        "permission_overwrites":         ("overwrites", _transform_overwrites),
        "splash_hash":                   ("splash", _guild_hash_transformer("splashes")),
        "banner_hash":                   ("banner", _guild_hash_transformer("banners")),
        "discovery_splash_hash":         ("discovery_splash", _guild_hash_transformer("discovery-splashes")),
        "icon_hash":                     ("icon", _transform_icon),
        "avatar_hash":                   ("avatar", _transform_avatar),
        "rate_limit_per_user":           ("slowmode_delay", None),
        "guild_id":                      ("guild", _transform_guild_id),
        "tags":                          ("emoji", None),
        "default_message_notifications": ("default_notifications", _enum_transformer(enums.NotificationLevel)),
        "region":                        (None, _enum_transformer(enums.VoiceRegion)),
        "rtc_region":                    (None, _enum_transformer(enums.VoiceRegion)),
        "video_quality_mode":            (None, _enum_transformer(enums.VideoQualityMode)),
        "privacy_level":                 (None, _enum_transformer(enums.StagePrivacyLevel)),
        "format_type":                   (None, _enum_transformer(enums.StickerFormatType)),
        "entity_type":                   (None, _enum_transformer(enums.ScheduledEventEntityType)),
        "type":                          (None, _transform_type),
        "trigger_type":                  (None, _enum_transformer(enums.AutoModerationTriggerType)),
        "event_type":                    (None, _enum_transformer(enums.AutoModerationEventType)),
        "actions":                       (None, _list_transformer(_transform_auto_moderation_action)),
        "trigger_metadata":              (None, _transform_auto_moderation_trigger_metadata),
        "exempt_roles":                  (None, _list_transformer(_transform_role)),
        "exempt_channels":               (None, _list_transformer(_transform_channel)),
    }
    # fmt: on

    def __init__(self, entry: AuditLogEntry, data: List[AuditLogChangePayload]):
        self.before = AuditLogDiff()
        self.after = AuditLogDiff()

        for elem in data:
            attr = elem["key"]

            # special cases for role add/remove
            if attr == "$add":
                self._handle_role(self.before, self.after, entry, elem["new_value"])  # type: ignore
                continue
            elif attr == "$remove":
                self._handle_role(self.after, self.before, entry, elem["new_value"])  # type: ignore
                continue

            try:
                key, transformer = self.TRANSFORMERS[attr]
            except (ValueError, KeyError):
                transformer = None
            else:
                if key:
                    attr = key

            transformer: Optional[Transformer]

            try:
                before = elem["old_value"]
            except KeyError:
                before = None
            else:
                if transformer:
                    before = transformer(entry, before)

            setattr(self.before, attr, before)

            try:
                after = elem["new_value"]
            except KeyError:
                after = None
            else:
                if transformer:
                    after = transformer(entry, after)

            setattr(self.after, attr, after)

        # add an alias
        if hasattr(self.after, "colour"):
            self.after.color = self.after.colour
            self.before.color = self.before.colour
        if hasattr(self.after, "expire_behavior"):
            self.after.expire_behaviour = self.after.expire_behavior
            self.before.expire_behaviour = self.before.expire_behavior

    def __repr__(self) -> str:
        return f"<AuditLogChanges before={self.before!r} after={self.after!r}>"

    def _handle_role(
        self,
        first: AuditLogDiff,
        second: AuditLogDiff,
        entry: AuditLogEntry,
        elem: List[RolePayload],
    ) -> None:
        if not hasattr(first, "roles"):
            setattr(first, "roles", [])

        data = []
        g: Guild = entry.guild

        for e in elem:
            role_id = int(e["id"])
            role = g.get_role(role_id)

            if role is None:
                role = Object(id=role_id)
                role.name = e["name"]  # type: ignore

            data.append(role)

        setattr(second, "roles", data)


class _AuditLogProxyMemberPrune:
    delete_member_days: int
    members_removed: int


class _AuditLogProxyMemberMoveOrMessageDelete:
    channel: abc.GuildChannel
    count: int


class _AuditLogProxyMemberDisconnect:
    count: int


class _AuditLogProxyPinAction:
    channel: abc.GuildChannel
    message_id: int


class _AuditLogProxyStageInstanceAction:
    channel: abc.GuildChannel


class _AuditLogProxyAutoModerationBlockMessage:
    channel: abc.GuildChannel
    rule_name: str
    rule_trigger_type: enums.AutoModerationTriggerType


class AuditLogEntry(Hashable):
    r"""Represents an Audit Log entry.

    You retrieve these via :meth:`Guild.audit_logs`.

    .. container:: operations

        .. describe:: x == y

            Checks if two entries are equal.

        .. describe:: x != y

            Checks if two entries are not equal.

        .. describe:: hash(x)

            Returns the entry's hash.

    .. versionchanged:: 1.7
        Audit log entries are now comparable and hashable.

    Attributes
    ----------
    action: :class:`AuditLogAction`
        The action that was done.
    user: :class:`abc.User`
        The user who initiated this action. Usually a :class:`Member`\, unless gone
        then it's a :class:`User`.
    id: :class:`int`
        The entry ID.
    target: Any
        The target that got changed. The exact type of this depends on
        the action being done.
    reason: Optional[:class:`str`]
        The reason this action was done.
    extra: Any
        Extra information that this entry has that might be useful.
        For most actions, this is ``None``. However in some cases it
        contains extra information. See :class:`AuditLogAction` for
        which actions have this field filled out.
    """
    extra: Union[
        _AuditLogProxyMemberPrune,
        _AuditLogProxyMemberMoveOrMessageDelete,
        _AuditLogProxyMemberDisconnect,
        _AuditLogProxyPinAction,
        _AuditLogProxyStageInstanceAction,
        _AuditLogProxyAutoModerationBlockMessage,
        Member,
        User,
        None,
        Role,
    ]

    def __init__(
        self,
        *,
        auto_moderation_rules: Dict[int, AutoModerationRule],
        users: Dict[int, User],
        data: AuditLogEntryPayload,
        guild: Guild,
    ):
        self._state = guild._state
        self.guild = guild
        self._auto_moderation_rules = auto_moderation_rules
        self._users = users
        self._from_data(data)

    def _from_data(self, data: AuditLogEntryPayload) -> None:
        self.action = enums.try_enum(enums.AuditLogAction, data["action_type"])
        self.id = int(data["id"])

        # this key is technically not usually present
        self.reason = data.get("reason")
        self.extra = data.get("options")  # type: ignore
        # I gave up trying to fix this

        if isinstance(self.action, enums.AuditLogAction) and self.extra:
            if self.action is enums.AuditLogAction.member_prune:
                # member prune has two keys with useful information
                self.extra = type(  # type: ignore
                    "_AuditLogProxy", (), {k: int(v) for k, v in self.extra.items()}  # type: ignore
                )()
            elif (
                self.action is enums.AuditLogAction.member_move
                or self.action is enums.AuditLogAction.message_delete
            ):
                channel_id = int(self.extra["channel_id"])
                elems = {
                    "count": int(self.extra["count"]),
                    "channel": self.guild.get_channel(channel_id) or Object(id=channel_id),
                }
                self.extra = type("_AuditLogProxy", (), elems)()  # type: ignore
            elif self.action is enums.AuditLogAction.member_disconnect:
                # The member disconnect action has a dict with some information
                elems = {
                    "count": int(self.extra["count"]),
                }
                self.extra = type("_AuditLogProxy", (), elems)()  # type: ignore
            elif self.action.name.endswith("pin"):
                # the pin actions have a dict with some information
                channel_id = int(self.extra["channel_id"])
                elems = {
                    "channel": self.guild.get_channel(channel_id) or Object(id=channel_id),
                    "message_id": int(self.extra["message_id"]),
                }
                self.extra = type("_AuditLogProxy", (), elems)()  # type: ignore
            elif self.action.name.startswith("overwrite_"):
                # the overwrite_ actions have a dict with some information
                instance_id = int(self.extra["id"])
                the_type = self.extra.get("type")
                if the_type == "1":
                    self.extra = self._get_member(instance_id)
                elif the_type == "0":
                    role = self.guild.get_role(instance_id)
                    if role is None:
                        role = Object(id=instance_id)
                        role.name = self.extra.get("role_name")  # type: ignore
                    self.extra = role  # type: ignore
            elif self.action.name.startswith("stage_instance"):
                channel_id = int(self.extra["channel_id"])
                elems = {"channel": self.guild.get_channel(channel_id) or Object(id=channel_id)}
                self.extra = type("_AuditLogProxy", (), elems)()  # type: ignore
            elif (
                self.action is enums.AuditLogAction.auto_moderation_block_message
                or self.action is enums.AuditLogAction.auto_moderation_flag_to_channel
                or self.action is enums.AuditLogAction.auto_moderation_user_communication_disabled
            ):
                channel_id = int(self.extra["channel_id"])
                elems = {
                    "channel": (
                        self.guild.get_channel_or_thread(channel_id) or Object(id=channel_id)
                    ),
                    "rule_name": self.extra["auto_moderation_rule_name"],
                    "rule_trigger_type": enums.try_enum(
                        enums.AutoModerationTriggerType,
                        int(self.extra["auto_moderation_rule_trigger_type"]),
                    ),
                }
                self.extra = type("_AuditLogProxy", (), elems)()  # type: ignore

        # this key is not present when the above is present, typically.
        # It's a list of { new_value: a, old_value: b, key: c }
        # where new_value and old_value are not guaranteed to be there depending
        # on the action type, so let's just fetch it for now and only turn it
        # into meaningful data when requested
        self._changes = data.get("changes", [])

        self.user = self._get_member(utils.get_as_snowflake(data, "user_id"))  # type: ignore
        self._target_id = utils.get_as_snowflake(data, "target_id")

    def _get_member(self, user_id: int) -> Union[Member, User, None]:
        return self.guild.get_member(user_id) or self._users.get(user_id)

    def __repr__(self) -> str:
        return f"<AuditLogEntry id={self.id} action={self.action} user={self.user!r}>"

    @utils.cached_property
    def created_at(self) -> datetime.datetime:
        """:class:`datetime.datetime`: Returns the entry's creation time in UTC."""
        return utils.snowflake_time(self.id)

    @utils.cached_property
    def target(self) -> AuditTarget:
        try:
            converter = getattr(self, "_convert_target_" + str(self.action.target_type))
        except AttributeError:
            if self._target_id is None:
                return None

            return Object(id=self._target_id)
        else:
            return converter(self._target_id)

    @utils.cached_property
    def category(self) -> Optional[enums.AuditLogActionCategory]:
        """Optional[:class:`AuditLogActionCategory`]: The category of the action, if applicable."""
        return self.action.category

    @utils.cached_property
    def changes(self) -> AuditLogChanges:
        """:class:`AuditLogChanges`: The list of changes this entry has."""
        obj = AuditLogChanges(self, self._changes)
        del self._changes
        return obj

    @utils.cached_property
    def before(self) -> AuditLogDiff:
        """:class:`AuditLogDiff`: The target's prior state."""
        return self.changes.before

    @utils.cached_property
    def after(self) -> AuditLogDiff:
        """:class:`AuditLogDiff`: The target's subsequent state."""
        return self.changes.after

    def _convert_target_guild(self, target_id: int) -> Guild:
        return self.guild

    def _convert_target_channel(self, target_id: int) -> Union[abc.GuildChannel, Object]:
        return self.guild.get_channel(target_id) or Object(id=target_id)

    def _convert_target_user(self, target_id: int) -> Union[Member, User, None]:
        return self._get_member(target_id)

    def _convert_target_role(self, target_id: int) -> Union[Role, Object]:
        return self.guild.get_role(target_id) or Object(id=target_id)

    def _convert_target_invite(self, target_id: int) -> Invite:
        # invites have target_id set to null
        # so figure out which change has the full invite data
        changeset = self.before if self.action is enums.AuditLogAction.invite_delete else self.after

        fake_payload = {
            "max_age": changeset.max_age,
            "max_uses": changeset.max_uses,
            "code": changeset.code,
            "temporary": changeset.temporary,
            "uses": changeset.uses,
        }

        obj = Invite(state=self._state, data=fake_payload, guild=self.guild, channel=changeset.channel)  # type: ignore
        try:
            obj.inviter = changeset.inviter
        except AttributeError:
            pass
        return obj

    def _convert_target_emoji(self, target_id: int) -> Union[Emoji, Object]:
        return self._state.get_emoji(target_id) or Object(id=target_id)

    def _convert_target_message(self, target_id: int) -> Union[Member, User, None]:
        return self._get_member(target_id)

    def _convert_target_stage_instance(self, target_id: int) -> Union[StageInstance, Object]:
        return self.guild.get_stage_instance(target_id) or Object(id=target_id)

    def _convert_target_sticker(self, target_id: int) -> Union[GuildSticker, Object]:
        return self._state.get_sticker(target_id) or Object(id=target_id)

    def _convert_target_thread(self, target_id: int) -> Union[Thread, Object]:
        return self.guild.get_thread(target_id) or Object(id=target_id)

    def _convert_target_auto_moderation_rule(
        self, rule_id: int
    ) -> Union[AutoModerationRule, Object]:
        return self._auto_moderation_rules.get(rule_id) or Object(id=rule_id)



# SPDX-License-Identifier: MIT

from __future__ import annotations

from typing import List, Literal, Optional, TypedDict, Union

from typing_extensions import NotRequired

from .auto_moderation import (
    AutoModerationAction,
    AutoModerationEventType,
    AutoModerationRule,
    AutoModerationTriggerMetadata,
    AutoModerationTriggerType,
)
from .channel import ChannelType, PermissionOverwrite, VideoQualityMode
from .guild import (
    DefaultMessageNotificationLevel,
    ExplicitContentFilterLevel,
    MFALevel,
    VerificationLevel,
)
from .integration import IntegrationExpireBehavior, PartialIntegration
from .role import Role
from .scheduled_events import ScheduledEventEntityType
from .snowflake import Snowflake
from .threads import Thread
from .user import User
from .webhook import Webhook

AuditLogEvent = Literal[
    1,
    10,
    11,
    12,
    13,
    14,
    15,
    20,
    21,
    22,
    23,
    24,
    25,
    26,
    27,
    28,
    30,
    31,
    32,
    40,
    41,
    42,
    50,
    51,
    52,
    60,
    61,
    62,
    72,
    73,
    74,
    75,
    80,
    81,
    82,
    83,
    84,
    85,
    90,
    91,
    92,
    100,
    101,
    102,
    110,
    111,
    112,
    140,
    141,
    142,
    143,
]


class _AuditLogChange_Str(TypedDict):
    key: Literal[
        "name",
        "description",
        "preferred_locale",
        "vanity_url_code",
        "topic",
        "code",
        "allow",
        "deny",
        "permissions",
        "tags",
    ]
    new_value: str
    old_value: str


class _AuditLogChange_AssetHash(TypedDict):
    key: Literal[
        "icon_hash", "splash_hash", "discovery_splash_hash", "banner_hash", "avatar_hash", "asset"
    ]
    new_value: str
    old_value: str


class _AuditLogChange_Snowflake(TypedDict):
    key: Literal[
        "id",
        "owner_id",
        "afk_channel_id",
        "rules_channel_id",
        "public_updates_channel_id",
        "widget_channel_id",
        "system_channel_id",
        "application_id",
        "channel_id",
        "inviter_id",
        "guild_id",
    ]
    new_value: Snowflake
    old_value: Snowflake


class _AuditLogChange_Bool(TypedDict):
    key: Literal[
        "widget_enabled",
        "nsfw",
        "hoist",
        "mentionable",
        "temporary",
        "deaf",
        "mute",
        "nick",
        "enabled_emoticons",
        "region",
        "rtc_region",
        "available",
        "archived",
        "locked",
        "enabled",
    ]
    new_value: bool
    old_value: bool


class _AuditLogChange_Int(TypedDict):
    key: Literal[
        "afk_timeout",
        "auto_archive_duration",
        "bitrate",
        "color",
        "default_auto_archive_duration",
        "max_age",
        "max_uses",
        "position",
        "prune_delete_days",
        "rate_limit_per_user",
        "user_limit",
    ]
    new_value: int
    old_value: int


class _AuditLogChange_ListRole(TypedDict):
    key: Literal["$add", "$remove"]
    new_value: List[Role]
    old_value: List[Role]


class _AuditLogChange_ListSnowflake(TypedDict):
    key: Literal["exempt_roles", "exempt_channels"]
    new_value: List[Snowflake]
    old_value: List[Snowflake]


class _AuditLogChange_MFALevel(TypedDict):
    key: Literal["mfa_level"]
    new_value: MFALevel
    old_value: MFALevel


class _AuditLogChange_VerificationLevel(TypedDict):
    key: Literal["verification_level"]
    new_value: VerificationLevel
    old_value: VerificationLevel


class _AuditLogChange_ExplicitContentFilter(TypedDict):
    key: Literal["explicit_content_filter"]
    new_value: ExplicitContentFilterLevel
    old_value: ExplicitContentFilterLevel


class _AuditLogChange_DefaultMessageNotificationLevel(TypedDict):
    key: Literal["default_message_notifications"]
    new_value: DefaultMessageNotificationLevel
    old_value: DefaultMessageNotificationLevel


class _AuditLogChange_ChannelType(TypedDict):
    key: Literal["type"]
    new_value: ChannelType
    old_value: ChannelType


class _AuditLogChange_IntegrationExpireBehaviour(TypedDict):
    key: Literal["expire_behavior"]
    new_value: IntegrationExpireBehavior
    old_value: IntegrationExpireBehavior


class _AuditLogChange_VideoQualityMode(TypedDict):
    key: Literal["video_quality_mode"]
    new_value: VideoQualityMode
    old_value: VideoQualityMode


class _AuditLogChange_Overwrites(TypedDict):
    key: Literal["permission_overwrites"]
    new_value: List[PermissionOverwrite]
    old_value: List[PermissionOverwrite]


class _AuditLogChange_ScheduledEventEntityType(TypedDict):
    key: Literal["entity_type"]
    new_value: ScheduledEventEntityType
    old_value: ScheduledEventEntityType


class _AuditLogChange_AutoModerationTriggerType(TypedDict):
    key: Literal["trigger_type"]
    new_value: AutoModerationTriggerType
    old_value: AutoModerationTriggerType


class _AuditLogChange_AutoModerationEventType(TypedDict):
    key: Literal["event_type"]
    new_value: AutoModerationEventType
    old_value: AutoModerationEventType


class _AuditLogChange_AutoModerationActions(TypedDict):
    key: Literal["actions"]
    new_value: List[AutoModerationAction]
    old_value: List[AutoModerationAction]


class _AuditLogChange_AutoModerationTriggerMetadata(TypedDict):
    key: Literal["trigger_metadata"]
    new_value: AutoModerationTriggerMetadata
    old_value: AutoModerationTriggerMetadata


AuditLogChange = Union[
    _AuditLogChange_AssetHash,
    _AuditLogChange_Bool,
    _AuditLogChange_ChannelType,
    _AuditLogChange_DefaultMessageNotificationLevel,
    _AuditLogChange_ScheduledEventEntityType,
    _AuditLogChange_ExplicitContentFilter,
    _AuditLogChange_Int,
    _AuditLogChange_IntegrationExpireBehaviour,
    _AuditLogChange_ListRole,
    _AuditLogChange_ListSnowflake,
    _AuditLogChange_MFALevel,
    _AuditLogChange_Overwrites,
    _AuditLogChange_Snowflake,
    _AuditLogChange_Str,
    _AuditLogChange_VerificationLevel,
    _AuditLogChange_VideoQualityMode,
    _AuditLogChange_AutoModerationTriggerType,
    _AuditLogChange_AutoModerationEventType,
    _AuditLogChange_AutoModerationActions,
    _AuditLogChange_AutoModerationTriggerMetadata,
]


class AuditEntryInfo(TypedDict):
    delete_member_days: str
    members_removed: str
    channel_id: Snowflake
    message_id: Snowflake
    count: str
    id: Snowflake
    type: Literal["0", "1"]
    role_name: str
    auto_moderation_rule_name: str
    auto_moderation_rule_trigger_type: str


class AuditLogEntry(TypedDict):
    target_id: Optional[str]
    user_id: Optional[Snowflake]
    id: Snowflake
    action_type: AuditLogEvent
    changes: NotRequired[List[AuditLogChange]]
    options: NotRequired[AuditEntryInfo]
    reason: NotRequired[str]


class AuditLog(TypedDict):
    webhooks: List[Webhook]
    users: List[User]
    audit_log_entries: List[AuditLogEntry]
    integrations: List[PartialIntegration]
    threads: List[Thread]
    auto_moderation_rules: List[AutoModerationRule]



# SPDX-License-Identifier: MIT

from __future__ import annotations

from typing import List, Literal, Optional, TypedDict, Union

from typing_extensions import NotRequired

from .emoji import DefaultReaction
from .snowflake import Snowflake
from .threads import ThreadArchiveDuration, ThreadMember, ThreadMetadata
from .user import PartialUser

OverwriteType = Literal[0, 1]
SortOrderType = Literal[0, 1]


class PermissionOverwrite(TypedDict):
    id: Snowflake
    type: OverwriteType
    allow: str
    deny: str


ChannelType = Literal[0, 1, 2, 3, 4, 5, 6, 10, 11, 12, 13]


class _BaseChannel(TypedDict):
    id: Snowflake
    name: str


class _BaseGuildChannel(_BaseChannel):
    guild_id: Snowflake
    position: int
    permission_overwrites: List[PermissionOverwrite]
    nsfw: bool
    parent_id: Optional[Snowflake]


class PartialChannel(_BaseChannel):
    type: ChannelType


class _ThreadedBaseChannel(_BaseGuildChannel, total=False):
    rate_limit_per_user: int
    default_auto_archive_duration: ThreadArchiveDuration
    default_thread_rate_limit_per_user: int


class TextChannel(_ThreadedBaseChannel):
    type: Literal[0]
    topic: NotRequired[str]
    last_message_id: NotRequired[Optional[Snowflake]]
    last_pin_timestamp: NotRequired[str]


class ForumChannel(_ThreadedBaseChannel):
    type: Literal[15]
    topic: NotRequired[str]
    last_message_id: NotRequired[Optional[Snowflake]]
    default_sort_order: Optional[SortOrderType]
    default_reaction_emoji: NotRequired[Optional[DefaultReaction]]
    default_thread_rate_limit_per_user: NotRequired[int]
    available_tags: NotRequired[List[ForumTag]]


class NewsChannel(_ThreadedBaseChannel):
    type: Literal[5]
    topic: NotRequired[str]
    last_message_id: NotRequired[Optional[Snowflake]]
    last_pin_timestamp: NotRequired[str]


VideoQualityMode = Literal[1, 2]


class VoiceChannel(_BaseGuildChannel):
    type: Literal[2]
    bitrate: int
    user_limit: int
    last_message_id: NotRequired[Optional[Snowflake]]
    rtc_region: NotRequired[Optional[str]]
    video_quality_mode: NotRequired[VideoQualityMode]


class CategoryChannel(_BaseGuildChannel):
    type: Literal[4]


class StageChannel(_BaseGuildChannel):
    type: Literal[13]
    bitrate: int
    user_limit: int
    rtc_region: NotRequired[Optional[str]]
    topic: NotRequired[str]


class ThreadChannel(_BaseChannel):
    type: Literal[10, 11, 12]
    guild_id: Snowflake
    parent_id: Snowflake
    owner_id: Snowflake
    nsfw: bool
    last_message_id: Optional[Snowflake]
    rate_limit_per_user: int
    message_count: int
    member_count: int
    thread_metadata: ThreadMetadata
    member: NotRequired[ThreadMember]
    last_pin_timestamp: NotRequired[str]


GuildChannel = Union[
    TextChannel,
    ForumChannel,
    NewsChannel,
    VoiceChannel,
    CategoryChannel,
    StageChannel,
    ThreadChannel,
]


class DMChannel(_BaseChannel):
    type: Literal[1]
    last_message_id: Optional[Snowflake]
    recipients: List[PartialUser]


class GroupDMChannel(_BaseChannel):
    type: Literal[3]
    icon: Optional[str]
    owner_id: Snowflake


Channel = Union[GuildChannel, DMChannel, GroupDMChannel]

PrivacyLevel = Literal[1, 2]


class StageInstance(TypedDict):
    id: Snowflake
    guild_id: Snowflake
    channel_id: Snowflake
    topic: str
    privacy_level: PrivacyLevel
    discoverable_disabled: bool


class ForumTag(TypedDict):
    id: Optional[Snowflake]
    name: str
    moderated: bool
    emoji_id: NotRequired[Optional[Snowflake]]
    emoji_name: NotRequired[Optional[str]]



# SPDX-License-Identifier: MIT

from __future__ import annotations

from typing import List, Literal, TypedDict, Union

from typing_extensions import NotRequired

from .channel import ChannelType
from .emoji import PartialEmoji

ComponentType = Literal[1, 2, 3, 4]
ButtonStyle = Literal[1, 2, 3, 4, 5]
TextInputStyle = Literal[1, 2]


class ActionRow(TypedDict):
    type: Literal[1]
    components: List[Component]


class ButtonComponent(TypedDict):
    type: Literal[2]
    style: ButtonStyle
    custom_id: NotRequired[str]
    url: NotRequired[str]
    disabled: NotRequired[bool]
    emoji: NotRequired[PartialEmoji]
    label: NotRequired[str]


class SelectOption(TypedDict):
    label: str
    value: str
    default: bool
    description: NotRequired[str]
    emoji: NotRequired[PartialEmoji]


class SelectMenuBase(TypedDict):
    custom_id: str
    placeholder: NotRequired[str]
    min_values: NotRequired[int]
    max_values: NotRequired[int]
    disabled: NotRequired[bool]


class SelectMenu(SelectMenuBase):
    type: Literal[3]
    options: List[SelectOption]


class UserSelectMenu(SelectMenuBase):
    type: Literal[5]


class RoleSelectMenu(SelectMenuBase):
    type: Literal[6]


class MentionableSelectMenu(SelectMenuBase):
    type: Literal[7]


class ChannelSelectMenu(SelectMenuBase):
    type: Literal[8]
    channel_types: NotRequired[List[ChannelType]]


class TextInputComponent(TypedDict):
    type: Literal[4]
    custom_id: str
    style: TextInputStyle
    label: str
    min_length: NotRequired[int]
    max_length: NotRequired[int]
    required: NotRequired[bool]
    value: NotRequired[str]
    placeholder: NotRequired[str]


Component = Union[ActionRow, ButtonComponent, SelectMenu, TextInputComponent]



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.interactions import (
    TYPE_CHECKING,
    ApplicationCommand,
    ApplicationCommandInteractionData,
    ApplicationCommandInteractionDataOption,
    ApplicationCommandInteractionDataResolved,
    ApplicationCommandOption,
    ApplicationCommandOptionChoice,
    ApplicationCommandOptionType,
    ApplicationCommandPermissions,
    ApplicationCommandPermissionType,
    ApplicationCommandResolvedPartialChannel,
    ApplicationCommandType,
    BaseGuildApplicationCommandPermissions,
    ChannelType,
    Component,
    ComponentInteractionData,
    ComponentType,
    Dict,
    EditApplicationCommand,
    Embed,
    GuildApplicationCommandPermissions,
    Interaction,
    InteractionApplicationCommandCallbackData,
    InteractionData,
    InteractionResponse,
    InteractionResponseType,
    InteractionType,
    List,
    Literal,
    Member,
    MessageInteraction,
    Optional,
    PartialGuildApplicationCommandPermissions,
    Role,
    Snowflake,
    TypedDict,
    Union,
    User,
    _ApplicationCommandInteractionDataOption,
    _ApplicationCommandInteractionDataOptional,
    _ApplicationCommandInteractionDataOptionBoolean,
    _ApplicationCommandInteractionDataOptionInteger,
    _ApplicationCommandInteractionDataOptionNumber,
    _ApplicationCommandInteractionDataOptionSnowflake,
    _ApplicationCommandInteractionDataOptionString,
    _ApplicationCommandInteractionDataOptionSubcommand,
    _ApplicationCommandOptional,
    _ApplicationCommandOptionOptional,
    _ComponentInteractionDataOptional,
    _EditApplicationCommandOptional,
    _InteractionOptional,
    _InteractionResponseOptional,
    annotations,
)



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.components import (
    ActionRow,
    ButtonComponent,
    ButtonStyle,
    Component,
    ComponentType,
    List,
    Literal,
    PartialEmoji,
    SelectMenu,
    SelectOption,
    TypedDict,
    Union,
    _ButtonComponentOptional,
    _SelectMenuOptional,
    _SelectOptionsOptional,
    annotations,
)



# SPDX-License-Identifier: MIT

from __future__ import annotations

import asyncio
import functools
from typing import TYPE_CHECKING, Callable, Union

import versacord
from versacord.application_command import (
    BaseApplicationCommand,
    CallbackWrapper,
    SlashApplicationSubcommand,
)
from versacord.interactions import Interaction

from .errors import (
    ApplicationBotMissingAnyRole,
    ApplicationBotMissingPermissions,
    ApplicationBotMissingRole,
    ApplicationCheckAnyFailure,
    ApplicationCheckFailure,
    ApplicationCheckForBotOnly,
    ApplicationMissingAnyRole,
    ApplicationMissingPermissions,
    ApplicationMissingRole,
    ApplicationNoPrivateMessage,
    ApplicationNotOwner,
    ApplicationNSFWChannelRequired,
    ApplicationPrivateMessageOnly,
)

if TYPE_CHECKING:
    from versacord.types.checks import ApplicationCheck, CoroFunc


__all__ = (
    "check",
    "check_any",
    "has_role",
    "has_any_role",
    "bot_has_role",
    "bot_has_any_role",
    "has_permissions",
    "bot_has_permissions",
    "has_guild_permissions",
    "bot_has_guild_permissions",
    "dm_only",
    "guild_only",
    "is_owner",
    "is_nsfw",
    "application_command_before_invoke",
    "application_command_after_invoke",
)


class CheckWrapper(CallbackWrapper):
    def __init__(self, callback: Union[Callable, CallbackWrapper], predicate):
        super().__init__(callback)

        if not asyncio.iscoroutinefunction(predicate):

            @functools.wraps(predicate)
            async def async_wrapper(ctx):
                return predicate(ctx)

            self.predicate = async_wrapper
        else:
            self.predicate = predicate

    def __call__(self, *args, **kwargs):
        return self.predicate(*args, **kwargs)

    def modify(self, app_cmd: BaseApplicationCommand):
        app_cmd.checks.append(self.predicate)


if TYPE_CHECKING:
    AC = Callable[
        [
            Union[
                CoroFunc,
                Callable[[Interaction], bool],
                BaseApplicationCommand,
                SlashApplicationSubcommand,
            ]
        ],
        CheckWrapper,
    ]


def check(predicate: "ApplicationCheck") -> AC:
    r"""A decorator that adds a check to the :class:`.BaseApplicationCommand` or its
    subclasses. These checks are accessible via :attr:`.BaseApplicationCommand.checks`.

    These checks should be predicates that take in a single parameter taking
    a :class:`.Interaction`. If the check returns a ``False``\-like value,
    an ApplicationCheckFailure is raised during invocation and sent to the
    :func:`.on_application_command_error` event.

    If an exception should be thrown in the predicate then it should be a
    subclass of :exc:`.ApplicationError`. Any exception not subclassed from it
    will be propagated while those subclassed will be sent to
    :func:`on_application_command_error`.

    A special attribute named ``predicate`` is bound to the value
    returned by this decorator to retrieve the predicate passed to the
    decorator. This allows the following introspection and chaining to be done:

    .. code-block:: python3

        def owner_or_permissions(**perms):
            original = application_checks.has_permissions(**perms).predicate
            async def extended_check(interaction: Interaction):
                if interaction.guild is None:
                    return False

                return (
                    interaction.guild.owner_id == interaction.user.id
                    or await original(interaction)
                )
            return application_checks.check(extended_check)

    .. note::

        The function returned by ``predicate`` is **always** a coroutine,
        even if the original function was not a coroutine.

    Examples
    --------

    Creating a basic check to see if the command invoker is you.

    .. code-block:: python3

        def check_if_it_is_me(interaction: Interaction):
            return interaction.message.author.id == 85309593344815104

        @bot.slash_command()
        @application_checks.check(check_if_it_is_me)
        async def only_for_me(interaction: Interaction):
            await interaction.response.send_message('I know you!')

    Transforming common checks into its own decorator:

    .. code-block:: python3

        def is_me():
            def predicate(interaction: Interaction):
                return interaction.user.id == 85309593344815104
            return application_checks.check(predicate)

        @bot.slash_command()
        @is_me()
        async def only_me(interaction: Interaction):
            await interaction.response.send_message('Only you!')

    Parameters
    ----------
    predicate: Callable[[:class:`~.Interaction`], :class:`bool`]
        The predicate to check if the command should be invoked.
    """

    def wrapper(func):
        return CheckWrapper(func, predicate)

    wrapper.predicate = predicate
    return wrapper


def check_any(*checks: "ApplicationCheck") -> AC:
    r"""A :func:`check` that will pass if any of the given checks pass,
    i.e. using logical OR.

    If all checks fail then :exc:`.ApplicationCheckAnyFailure` is raised to signal
    the failure. It inherits from :exc:`.ApplicationCheckFailure`.

    .. note::

        The ``predicate`` attribute for this function **is** a coroutine.

    Parameters
    ----------
    \*checks: Callable[[:class:`~.Interaction`], :class:`bool`]
        An argument list of checks that have been decorated with
        the :func:`check` decorator.

    Raises
    ------
    TypeError
        A check passed has not been decorated with the :func:`check`
        decorator.

    Examples
    --------

    Creating a basic check to see if it's the bot owner or
    the server owner:

    .. code-block:: python3

        def is_guild_owner():
            def predicate(interaction: Interaction):
                return (
                    interaction.guild is not None
                    and interaction.guild.owner_id == interaction.user.id
                )
            return application_checks.check(predicate)

        @bot.slash_command()
        @application_checks.check_any(applications_checks.is_owner(), is_guild_owner())
        async def only_for_owners(interaction: Interaction):
            await interaction.response.send_message('Hello mister owner!')
    """

    unwrapped = []
    for wrapped in checks:
        try:
            # we only want to get the predicate, the arg type is not used
            wrapper = wrapped(None)  # type: ignore
            pred = wrapper.predicate  # type: ignore
        except AttributeError:
            raise TypeError(
                f"{wrapped!r} must be wrapped by application_checks.check decorator"
            ) from None
        else:
            unwrapped.append(pred)

    async def predicate(interaction: Interaction) -> bool:
        errors = []
        for func in unwrapped:
            try:
                value = await func(interaction)
            except ApplicationCheckFailure as e:
                errors.append(e)
            else:
                if value:
                    return True
        # if we're here, all checks failed
        raise ApplicationCheckAnyFailure(unwrapped, errors)

    return check(predicate)


def has_role(item: Union[int, str]) -> AC:
    """A :func:`.check` that is added that checks if the member invoking the
    command has the role specified via the name or ID specified.

    If a string is specified, you must give the exact name of the role, including
    caps and spelling.

    If an integer is specified, you must give the exact snowflake ID of the role.

    If the message is invoked in a private message context then the check will
    return ``False``.

    This check raises one of two special exceptions, :exc:`.MissingRole` if the user
    is missing a role, or :exc:`.NoPrivateMessage` if it is used in a private message.
    Both inherit from :exc:`.ApplicationCheckFailure`.

    Parameters
    ----------
    item: Union[:class:`int`, :class:`str`]
        The name or ID of the role to check.

    Example
    -------

    .. code-block:: python3

        @bot.slash_command()
        @application_checks.has_role('Cool Role')
        async def cool(interaction: Interaction):
            await interaction.response.send_message('You are cool indeed')
    """

    def predicate(interaction: Interaction) -> bool:
        if interaction.guild is None:
            raise ApplicationNoPrivateMessage()

        # interaction.guild is None doesn't narrow interaction.user to Member
        if isinstance(item, int):
            role = versacord.utils.get(interaction.user.roles, id=item)  # type: ignore
        else:
            role = versacord.utils.get(interaction.user.roles, name=item)  # type: ignore
        if role is None:
            raise ApplicationMissingRole(item)
        return True

    return check(predicate)


def has_any_role(*items: Union[int, str]) -> AC:
    r"""A :func:`.check` that is added that checks if the member invoking the
    command has **any** of the roles specified. This means that if they have
    one out of the three roles specified, then this check will return ``True``.

    Similar to :func:`.has_role`\, the names or IDs passed in must be exact.

    This check raises one of two special exceptions, :exc:`.MissingAnyRole` if the user
    is missing all roles, or :exc:`.NoPrivateMessage` if it is used in a private message.
    Both inherit from :exc:`.ApplicationCheckFailure`.

    Parameters
    ----------
    items: List[Union[:class:`str`, :class:`int`]]
        An argument list of names or IDs to check that the member has roles wise.

    Example
    -------

    .. code-block:: python3

        @bot.slash_command()
        @application_checks.has_any_role('Moderators', 492212595072434186)
        async def cool(interaction: Interaction):
            await interaction.response.send_message('You are cool indeed')
    """

    def predicate(interaction: Interaction) -> bool:
        if interaction.guild is None:
            raise ApplicationNoPrivateMessage()

        # interaction.guild is None doesn't narrow interaction.user to Member
        getter = functools.partial(versacord.utils.get, interaction.user.roles)  # type: ignore
        if any(
            getter(id=item) is not None if isinstance(item, int) else getter(name=item) is not None
            for item in items
        ):
            return True
        raise ApplicationMissingAnyRole(list(items))

    return check(predicate)


def bot_has_role(item: Union[int, str]) -> AC:
    """Similar to :func:`.has_role` except checks if the bot itself has the
    role.

    This check raises one of two special exceptions,
    :exc:`.ApplicationBotMissingRole` if the bot is missing the role,
    or :exc:`.ApplicationNoPrivateMessage` if it is used in a private message.
    Both inherit from :exc:`.ApplicationCheckFailure`.

    Parameters
    ----------
    item: Union[:class:`int`, :class:`str`]
        The name or ID of the role to check.

    Example
    -------

    .. code-block:: python3

        @bot.slash_command()
        @application_checks.bot_has_role(492212595072434186)
        async def hasrole(interaction: Interaction):
            await interaction.response.send_message('I have the required role!')
    """

    def predicate(interaction: Interaction) -> bool:
        if interaction.guild is None:
            raise ApplicationNoPrivateMessage()

        me = interaction.guild.me
        if isinstance(item, int):
            role = versacord.utils.get(me.roles, id=item)
        else:
            role = versacord.utils.get(me.roles, name=item)
        if role is None:
            raise ApplicationBotMissingRole(item)
        return True

    return check(predicate)


def bot_has_any_role(*items: Union[str, int]) -> AC:
    """Similar to :func:`.has_any_role` except checks if the bot itself has
    any of the roles listed.

    This check raises one of two special exceptions,
    :exc:`.ApplicationBotMissingAnyRole` if the bot is missing all roles,
    or :exc:`.ApplicationNoPrivateMessage` if it is used in a private message.
    Both inherit from :exc:`.ApplicationCheckFailure`.

    Parameters
    ----------
    *items: Union[:class:`str`, :class:`int`]
        An argument list of names or IDs to check that the bot has roles wise.

    Example
    -------

    .. code-block:: python3

        @bot.slash_command()
        @application_checks.bot_has_any_role('Moderators', 492212595072434186)
        async def cool(interaction: Interaction):
            await interaction.response.send_message('I have a required role!')
    """

    def predicate(interaction: Interaction) -> bool:
        if interaction.guild is None:
            raise ApplicationNoPrivateMessage()

        getter = functools.partial(versacord.utils.get, interaction.guild.me.roles)
        if any(
            getter(id=item) is not None if isinstance(item, int) else getter(name=item) is not None
            for item in items
        ):
            return True
        raise ApplicationBotMissingAnyRole(list(items))

    return check(predicate)


def has_permissions(**perms: bool) -> AC:
    """A :func:`.check` that is added that checks if the member has all of
    the permissions necessary.

    Note that this check operates on the current channel permissions, not the
    guild wide permissions.

    The permissions passed in must be exactly like the properties shown under
    :class:`.versacord.Permissions`.

    This check raises a special exception, :exc:`.ApplicationMissingPermissions`
    that is inherited from :exc:`.ApplicationCheckFailure`.

    If this check is called in a DM context, it will raise an
    exception, :exc:`.ApplicationNoPrivateMessage`.

    Parameters
    ----------
    perms: :class:`bool`
        An argument list of permissions to check for.

    Example
    -------

    .. code-block:: python3

        @bot.slash_command()
        @application_checks.has_permissions(manage_messages=True)
        async def testperms(interaction: Interaction):
            await interaction.response.send_message('You can manage messages.')

    """

    invalid = set(perms) - set(versacord.Permissions.VALID_FLAGS)
    if invalid:
        raise TypeError(f"Invalid permission(s): {', '.join(invalid)}")

    def predicate(interaction: Interaction) -> bool:
        ch = interaction.channel
        try:
            permissions = ch.permissions_for(interaction.user)  # type: ignore
        except AttributeError:
            raise ApplicationNoPrivateMessage()

        missing = [perm for perm, value in perms.items() if getattr(permissions, perm) != value]

        if not missing:
            return True

        raise ApplicationMissingPermissions(missing)

    return check(predicate)


def bot_has_permissions(**perms: bool) -> AC:
    """Similar to :func:`.has_permissions` except checks if the bot itself has
    the permissions listed.

    This check raises a special exception, :exc:`.ApplicationBotMissingPermissions`
    that is inherited from :exc:`.ApplicationCheckFailure`.

    If this check is called in a DM context, it will raise an
    exception, :exc:`.ApplicationNoPrivateMessage`.
    """

    invalid = set(perms) - set(versacord.Permissions.VALID_FLAGS)
    if invalid:
        raise TypeError(f"Invalid permission(s): {', '.join(invalid)}")

    def predicate(interaction: Interaction) -> bool:
        guild = interaction.guild
        me = guild.me if guild is not None else interaction.client.user
        ch = interaction.channel
        try:
            permissions = ch.permissions_for(me)  # type: ignore
        except AttributeError:
            raise ApplicationNoPrivateMessage()

        missing = [perm for perm, value in perms.items() if getattr(permissions, perm) != value]

        if not missing:
            return True

        raise ApplicationBotMissingPermissions(missing)

    return check(predicate)


def has_guild_permissions(**perms: bool) -> AC:
    """Similar to :func:`.has_permissions`, but operates on guild wide
    permissions instead of the current channel permissions.

    If this check is called in a DM context, it will raise an
    exception, :exc:`.ApplicationNoPrivateMessage`.

    Parameters
    ----------
    perms: :class:`bool`
        An argument list of guild permissions to check for.

    Example
    -------

    .. code-block:: python3

        @bot.slash_command()
        @application_checks.has_guild_permissions(manage_messages=True)
        async def permcmd(interaction: Interaction):
            await interaction.response.send_message('You can manage messages!')
    """

    invalid = set(perms) - set(versacord.Permissions.VALID_FLAGS)
    if invalid:
        raise TypeError(f"Invalid permission(s): {', '.join(invalid)}")

    def predicate(interaction: Interaction) -> bool:
        if not interaction.guild:
            raise ApplicationNoPrivateMessage

        permissions = interaction.user.guild_permissions  # type: ignore
        missing = [perm for perm, value in perms.items() if getattr(permissions, perm) != value]

        if not missing:
            return True

        raise ApplicationMissingPermissions(missing)

    return check(predicate)


def bot_has_guild_permissions(**perms: bool) -> AC:
    """Similar to :func:`.has_guild_permissions`, but checks the bot
    members guild permissions.
    """

    invalid = set(perms) - set(versacord.Permissions.VALID_FLAGS)
    if invalid:
        raise TypeError(f"Invalid permission(s): {', '.join(invalid)}")

    def predicate(interaction: Interaction) -> bool:
        if not interaction.guild:
            raise ApplicationNoPrivateMessage

        permissions = interaction.guild.me.guild_permissions
        missing = [perm for perm, value in perms.items() if getattr(permissions, perm) != value]

        if not missing:
            return True

        raise ApplicationBotMissingPermissions(missing)

    return check(predicate)


def dm_only() -> AC:
    """A :func:`.check` that indicates this command must only be used in a
    DM context. Only private messages are allowed when
    using the command.

    This check raises a special exception, :exc:`.ApplicationPrivateMessageOnly`
    that is inherited from :exc:`.ApplicationCheckFailure`.

    Example
    -------

    .. code-block:: python3

        @bot.slash_command()
        @application_checks.dm_only()
        async def dmcmd(interaction: Interaction):
            await interaction.response.send_message('This is in DMS!')
    """

    def predicate(interaction: Interaction) -> bool:
        if interaction.guild is not None:
            raise ApplicationPrivateMessageOnly()
        return True

    return check(predicate)


def guild_only() -> AC:
    """A :func:`.check` that indicates this command must only be used in a
    guild context only. Basically, no private messages are allowed when
    using the command.

    This check raises a special exception, :exc:`.ApplicationNoPrivateMessage`
    that is inherited from :exc:`.ApplicationCheckFailure`.

    Example
    -------

    .. code-block:: python3

        @bot.slash_command()
        @application_checks.guild_only()
        async def dmcmd(interaction: Interaction):
            await interaction.response.send_message('This is in a GUILD!')
    """

    def predicate(interaction: Interaction) -> bool:
        if interaction.guild is None:
            raise ApplicationNoPrivateMessage()
        return True

    return check(predicate)


def is_owner() -> AC:
    """A :func:`.check` that checks if the person invoking this command is the
    owner of the bot.

    This is powered by :meth:`.ext.commands.Bot.is_owner`.

    This check raises a special exception, :exc:`.ApplicationNotOwner` that is derived
    from :exc:`.ApplicationCheckFailure`.

    This check may only be used with :class:`~ext.commands.Bot`. Otherwise, it will
    raise :exc:`.ApplicationCheckForBotOnly`.

    Example
    -------

    .. code-block:: python3

        bot = commands.Bot(owner_id=297045071457681409)

        @bot.slash_command()
        @application_checks.is_owner()
        async def ownercmd(interaction: Interaction):
            await interaction.response.send_message('Only you!')
    """

    async def predicate(interaction: Interaction) -> bool:
        if not hasattr(interaction.client, "is_owner"):
            raise ApplicationCheckForBotOnly()

        if not await interaction.client.is_owner(interaction.user):
            raise ApplicationNotOwner("You do not own this bot.")
        return True

    return check(predicate)


def is_nsfw() -> AC:
    """A :func:`.check` that checks if the channel is a NSFW channel.

    This check raises a special exception, :exc:`.ApplicationNSFWChannelRequired`
    that is derived from :exc:`.ApplicationCheckFailure`.

    Example
    -------

    .. code-block:: python3

        @bot.slash_command()
        @application_checks.is_nsfw()
        async def ownercmd(interaction: Interaction):
            await interaction.response.send_message('Only NSFW channels!')
    """

    def pred(interaction: Interaction) -> bool:
        ch = interaction.channel
        if interaction.guild is None or (
            isinstance(ch, (versacord.TextChannel, versacord.Thread)) and ch.is_nsfw()
        ):
            return True
        raise ApplicationNSFWChannelRequired(ch)

    return check(pred)


def application_command_before_invoke(coro) -> AC:
    """A decorator that registers a coroutine as a pre-invoke hook.

    This allows you to refer to one before invoke hook for several commands that
    do not have to be within the same cog.

    Example
    -------

    .. code-block:: python3

        async def record_usage(interaction: Interaction):
            print(
                interaction.user,
                "used",
                interaction.application_command,
                "at",
                interaction.message.created_at
            )

        @bot.slash_command()
        @application_checks.application_command_before_invoke(record_usage)
        async def who(interaction: Interaction): # Output: <User> used who at <Time>
            await interaction.response.send_message("I am a bot")

        class What(commands.Cog):
            @application_checks.application_command_before_invoke(record_usage)
            @versacord.slash_command()
            async def when(self, interaction: Interaction):
                # Output: <User> used when at <Time>
                await interaction.response.send_message(
                    f"and i have existed since {interaction.client.user.created_at}"
                )

            @versacord.slash_command()
            async def where(self, interaction: Interaction): # Output: <Nothing>
                await interaction.response.send_message("on Discord")

            @versacord.slash_command()
            async def why(self, interaction: Interaction): # Output: <Nothing>
                await interaction.response.send_message("because someone made me")

        bot.add_cog(What())
    """

    class BeforeInvokeModifier(CallbackWrapper):
        def modify(self, app_cmd: BaseApplicationCommand):
            app_cmd._callback_before_invoke = coro

    def decorator(
        func: Union[SlashApplicationSubcommand, BaseApplicationCommand, "CoroFunc"]
    ) -> Union[SlashApplicationSubcommand, BaseApplicationCommand, BeforeInvokeModifier]:
        return BeforeInvokeModifier(func)

    return decorator  # type: ignore


def application_command_after_invoke(coro) -> AC:
    """A decorator that registers a coroutine as a post-invoke hook.

    This allows you to refer to one after invoke hook for several commands that
    do not have to be within the same cog.
    """

    class AfterInvokeModifier(CallbackWrapper):
        def modify(self, app_cmd: BaseApplicationCommand):
            app_cmd._callback_after_invoke = coro

    def decorator(
        func: Union[SlashApplicationSubcommand, BaseApplicationCommand, "CoroFunc"]
    ) -> Union[SlashApplicationSubcommand, BaseApplicationCommand, AfterInvokeModifier]:
        return AfterInvokeModifier(func)

    return decorator  # type: ignore



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ui.button import (
    TYPE_CHECKING,
    B,
    Button,
    ButtonComponent,
    ButtonStyle,
    Callable,
    ComponentType,
    Item,
    ItemCallbackType,
    Optional,
    PartialEmoji,
    Tuple,
    Type,
    TypeVar,
    Union,
    V,
    _EmojiTag,
    annotations,
    button,
    inspect,
    os,
)

__all__ = ("Button", "button")



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.file import TYPE_CHECKING, File, Optional, Union, annotations, io, os

__all__ = "File"



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.permissions import (
    PO,
    TYPE_CHECKING,
    Any,
    BaseFlags,
    Callable,
    ClassVar,
    Dict,
    Iterator,
    Optional,
    P,
    PermissionOverwrite,
    Permissions,
    Set,
    Tuple,
    Type,
    TypeVar,
    _augment_from_permissions,
    alias_flag_value,
    annotations,
    fill_with_flags,
    flag_value,
    make_permission_alias,
    permission_alias,
)

__all__ = ("Permissions", "PermissionOverwrite")



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.abc import (
    GCH,
    MISSING,
    TYPE_CHECKING,
    AllowedMentions,
    Any,
    Callable,
    ChannelType,
    ClientException,
    Connectable,
    Dict,
    File,
    GuildChannel,
    GuildSticker,
    HistoryIterator,
    InvalidArgument,
    Invite,
    List,
    Messageable,
    Optional,
    PermissionOverwrite,
    Permissions,
    PrivateChannel,
    Protocol,
    Role,
    Sequence,
    Snowflake,
    StickerItem,
    T,
    Tuple,
    TypeVar,
    Typing,
    Union,
    User,
    VoiceClient,
    VoiceProtocol,
    _Overwrites,
    _Undefined,
    _undefined,
    annotations,
    asyncio,
    copy,
    overload,
    runtime_checkable,
    utils,
)

__all__ = ("Snowflake", "User", "PrivateChannel", "GuildChannel", "Messageable", "Connectable")



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.application_command import (
    MISSING,
    TYPE_CHECKING,
    Any,
    ApplicationCheckFailure,
    ApplicationCommand,
    ApplicationCommandOptionType,
    ApplicationCommandType,
    ApplicationError,
    ApplicationInvokeError,
    ApplicationSubcommand,
    Attachment,
    Callable,
    ChannelType,
    ClientCog,
    CommandOption,
    Dict,
    FuncT,
    Guild,
    GuildChannel,
    Interaction,
    InvalidCommandType,
    Iterable,
    List,
    Member,
    Message,
    Optional,
    Parameter,
    Role,
    Set,
    SlashOption,
    T,
    Tuple,
    TypeVar,
    Union,
    User,
    _cog_special_method,
    annotations,
    asyncio,
    check_dictionary_values,
    deep_dictionary_check,
    find,
    maybe_coroutine,
    message_command,
    signature,
    slash_command,
    typing,
    user_command,
)

__all__ = (
    "ApplicationCommand",
    "ApplicationSubcommand",
    "ClientCog",
    "CommandOption",
    "message_command",
    "SlashOption",
    "slash_command",
    "user_command",
)



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.embeds import (
    TYPE_CHECKING,
    Any,
    Colour,
    Dict,
    E,
    Embed,
    EmbedProxy,
    EmptyEmbed,
    Final,
    List,
    Mapping,
    Protocol,
    Type,
    TypeVar,
    Union,
    _EmptyEmbed,
    annotations,
    datetime,
    utils,
)

__all__ = "Embed"



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ui.item import (
    TYPE_CHECKING,
    Any,
    Callable,
    Coroutine,
    Dict,
    Generic,
    I,
    Interaction,
    Item,
    ItemCallbackType,
    Optional,
    Tuple,
    Type,
    TypeVar,
    V,
    annotations,
)

__all__ = "Item"



# SPDX-License-Identifier: MIT

from warnings import warn

from pkg_resources import DistributionNotFound, get_distribution

__all__ = ("incompatible_libraries",)


class DistributionWarning(RuntimeWarning):
    ...


incompatible_libraries = ["discord.py", "discord", "pyfork", "enhanced-dpy", "py-cord"]

for library in incompatible_libraries:
    try:
        get_distribution(library)
        # Library is installed. Throw a warning
        message = (
            f"{library} is installed which is incompatible with versacord. "
            f"Please remove this library by using `pip3 uninstall {library}`"
        )

        warn(message, DistributionWarning, stacklevel=0)
    except DistributionNotFound:
        pass



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.http import (
    MISSING,
    TYPE_CHECKING,
    Any,
    ClassVar,
    Coroutine,
    Dict,
    DiscordClientWebSocketResponse,
    DiscordServerError,
    Forbidden,
    GatewayNotFound,
    HTTPClient,
    HTTPException,
    InvalidArgument,
    Iterable,
    List,
    LoginFailure,
    MaybeUnlock,
    NotFound,
    Optional,
    Route,
    Sequence,
    Tuple,
    Type,
    TypeVar,
    Union,
    _log,
    _uriquote,
    aiohttp,
    annotations,
    asyncio,
    json,
    json_or_text,
    logging,
    sys,
    utils,
    weakref,
)



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.webhook.__init__ import (
    PartialWebhookChannel,
    PartialWebhookGuild,
    SyncWebhook,
    SyncWebhookMessage,
    Webhook,
    WebhookMessage,
)



# SPDX-License-Identifier: MIT

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Optional, Tuple, TypeVar

from ..components import TextInput as TextInputComponent
from ..enums import ComponentType, TextInputStyle
from ..guild import Guild
from ..state import ConnectionState
from ..utils import MISSING
from .item import Item

__all__ = ("TextInput",)

if TYPE_CHECKING:
    from typing_extensions import Self

    from ..types.components import TextInputComponent as TextInputComponentPayload
    from ..types.interactions import ComponentInteractionData
    from .view import View


V = TypeVar("V", bound="View", covariant=True)


class TextInput(Item[V]):
    """Represent a UI text input.

    .. versionadded:: 2.0

    Parameters
    ----------
    label: :class:`str`
        The label of the text input
    style: :class:`TextInputStyle`
        The style of the text input.
        By default, the style is ``TextInputStyle.short``, a one line
        input, but you can use ``TextInputStyle.paragraph``, a multi line
        input.
    custom_id: Optional[:class:`str`]
        The ID of the text input that get received during an interaction.
    row: Optional[:class:`int`]
        The relative row this text input belongs to. A Discord component can only have 5
        rows. By default, items are arranged automatically into those 5 rows. If you'd
        like to control the relative positioning of the row then passing an index is advised.
        For example, row=1 will show up before row=2. Defaults to ``None``, which is automatic
        ordering. The row number must be between 0 and 4 (i.e. zero indexed).
    min_length: Optional[:class:`int`]
        The minimal length of the user's input
    max_length: Optional[:class:`int`]
        The maximal length of the user's input
    required: Optional[:class:`bool`]
        If ``True``, the user cannot send the form without filling this
        field.
    default_value: Optional[:class:`str`]
        The value already in the input when the user open the form.
    placeholder: Optional[:class:`str`]
        The text shown to the user when the text input is empty.
    """

    __item_repr_attributes__: Tuple[str, ...] = (
        "custom_id",
        "label",
        "style",
        "min_length",
        "max_length",
        "required",
        "default_value",
        "value",
        "placeholder",
    )

    def __init__(
        self,
        label: str,
        *,
        style: TextInputStyle = TextInputStyle.short,
        custom_id: str = MISSING,
        row: Optional[int] = None,
        min_length: Optional[int] = 0,
        max_length: Optional[int] = 4000,
        required: Optional[bool] = None,
        default_value: Optional[str] = None,
        placeholder: Optional[str] = None,
    ):
        self._provided_custom_id = custom_id is not MISSING
        custom_id = os.urandom(16).hex() if custom_id is MISSING else custom_id
        self._underlying = TextInputComponent._raw_construct(
            type=ComponentType.text_input,
            custom_id=custom_id,
            label=label,
            style=style,
            min_length=min_length,
            max_length=max_length,
            required=required,
            value=default_value,
            placeholder=placeholder,
        )
        self.row = row
        self._inputed_value: Optional[str] = None

    @property
    def style(self) -> TextInputStyle:
        """:class:`versacord.TextInputStyle`: The style of the text input."""
        return self._underlying.style

    @style.setter
    def style(self, value: TextInputStyle) -> None:
        self._underlying.style = value

    @property
    def custom_id(self) -> Optional[str]:
        """Optional[:class:`str`]: The ID of the text input that gets received during an interaction."""
        return self._underlying.custom_id

    @custom_id.setter
    def custom_id(self, value: Optional[str]) -> None:
        if value is not None and not isinstance(value, str):
            raise TypeError("custom_id must be None or str")

        self._underlying.custom_id = value  # type: ignore

    @property
    def label(self) -> str:
        """:class:`str`: The label of the text input."""
        return self._underlying.label

    @label.setter
    def label(self, value: str) -> None:
        if value is None:
            raise TypeError("label must cannot be None")
        self._underlying.label = str(value)

    @property
    def min_length(self) -> Optional[int]:
        """:class:`int`: The minimum input length for a text input"""
        return self._underlying.min_length

    @min_length.setter
    def min_length(self, value: int) -> None:
        self._underlying.min_length = value

    @property
    def max_length(self) -> Optional[int]:
        """:class:`int`: The maximum input length for a text input"""
        return self._underlying.max_length

    @max_length.setter
    def max_length(self, value: int) -> None:
        self._underlying.max_length = value

    @property
    def required(self) -> Optional[bool]:
        """:class:`bool`: Whether this component is required to be filled"""
        return self._underlying.required

    @required.setter
    def required(self, value: Optional[bool]) -> None:
        if value is not None and not isinstance(value, bool):
            raise TypeError("required must be None or bool")

        self._underlying.required = value

    @property
    def default_value(self) -> Optional[str]:
        """Optional[:class:`str`]: The value already in the text input when the user open the form."""
        return self._underlying.value

    @default_value.setter
    def default_value(self, value: Optional[str]) -> None:
        if value is not None and not isinstance(value, str):
            raise TypeError("default_value must be None or str")
        self._underlying.value = value

    @property
    def value(self) -> Optional[str]:
        """Optional[:class:`str`]: The value sent by the user.
        This field is updated when an interaction is received.
        ``TextInput.value`` is ``None`` when no interaction where received.
        """
        return self._inputed_value

    @property
    def placeholder(self) -> Optional[str]:
        """Optional[:class:`str`]: The text shown to the user when the text input is empty."""
        return self._underlying.placeholder

    @placeholder.setter
    def placeholder(self, value: Optional[str]) -> None:
        if value is not None and not isinstance(value, str):
            raise TypeError("placeholder must be None or str")

        self._underlying.placeholder = value

    @property
    def width(self) -> int:
        return 5

    @classmethod
    def from_component(cls, text_input: TextInputComponent) -> Self:
        return cls(
            style=text_input.style,
            custom_id=text_input.custom_id,
            label=text_input.label,
            min_length=text_input.min_length,
            max_length=text_input.max_length,
            required=text_input.required,
            # value=text_input.value,  # FIXME: figure out what this was
            placeholder=text_input.placeholder,
            row=None,
        )

    @property
    def type(self) -> ComponentType:
        return self._underlying.type

    def to_component_dict(self) -> TextInputComponentPayload:
        return self._underlying.to_dict()

    def is_dispatchable(self) -> bool:
        return True

    def refresh_component(self, text_input: TextInputComponent) -> None:
        self._underlying = text_input

    def refresh_state(
        self, data: ComponentInteractionData, state: ConnectionState, guild: Optional[Guild]
    ) -> None:
        self._inputed_value = data.get("value", "")



# SPDX-License-Identifier: MIT
"""Module to allow for backwards compatibility for existing code and extensions."""

from versacord import *

__title__ = "versacord framework"
__author__ = "versacord Developers"
__license__ = "MIT"
__copyright__ = "Copyright 022-present versacord Developers"
__version__ = "1.0.0"

__path__ = __import__("pkgutil").extend_path(__path__, __name__)



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.colour import (
    CT,
    Any,
    Color,
    Colour,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
    colorsys,
    random,
)

__all__ = ("Colour", "Color")



# SPDX-License-Identifier: MIT

from typing import Literal, TypedDict

from .member import Member
from .snowflake import Snowflake
from .user import User

ScheduledEventEntityType = Literal[1, 2, 3]
ScheduledEventPrivacyLevel = Literal[2]
ScheduledEventStatus = Literal[1, 2, 3, 4]


class EntityMetadata(TypedDict, total=False):
    location: str


class ScheduledEvent(TypedDict):
    id: Snowflake
    guild_id: Snowflake
    channel_id: Snowflake
    name: str
    description: str
    scheduled_start_time: str
    scheduled_end_time: str
    privacy_level: ScheduledEventPrivacyLevel
    status: ScheduledEventStatus
    entity_type: ScheduledEventEntityType
    entity_id: Snowflake
    entity_metadata: EntityMetadata
    creator: User
    user_count: int
    image: str


class ScheduledEventUser(TypedDict):
    guild_scheduled_event_id: Snowflake
    user: User
    member: Member



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.asset import (
    MISSING,
    TYPE_CHECKING,
    VALID_ASSET_FORMATS,
    VALID_STATIC_FORMATS,
    Any,
    Asset,
    AssetMixin,
    DiscordException,
    InvalidArgument,
    Literal,
    Optional,
    Tuple,
    Union,
    annotations,
    io,
    os,
    utils,
    yarl,
)

__all__ = "Asset"



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands._types import (
    TYPE_CHECKING,
    Any,
    Callable,
    Check,
    Coro,
    CoroFunc,
    Coroutine,
    Error,
    Hook,
    MaybeCoro,
    T,
    TypeVar,
    Union,
    _BaseCommand,
)



# SPDX-License-Identifier: MIT

from __future__ import annotations

from typing import List, Optional, TypedDict

from typing_extensions import NotRequired

from .snowflake import Snowflake
from .team import Team
from .user import User


class BaseAppInfo(TypedDict):
    id: Snowflake
    name: str
    verify_key: str
    icon: Optional[str]
    summary: str
    description: str


class AppInfo(BaseAppInfo):
    rpc_origins: List[str]
    owner: User
    bot_public: bool
    bot_require_code_grant: bool
    team: NotRequired[Team]
    terms_of_service_url: NotRequired[str]
    privacy_policy_url: NotRequired[str]
    hook: NotRequired[bool]
    max_participants: NotRequired[int]


class PartialAppInfo(BaseAppInfo, total=False):
    rpc_origins: List[str]
    cover_image: str
    hook: bool
    terms_of_service_url: str
    privacy_policy_url: str
    max_participants: int
    flags: int



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.gateway import (
    BaseActivity,
    ConnectionClosed,
    DiscordClientWebSocketResponse,
    DiscordVoiceWebSocket,
    DiscordWebSocket,
    EventListener,
    GatewayRatelimiter,
    InvalidArgument,
    KeepAliveHandler,
    ReconnectWebSocket,
    SpeakingState,
    VoiceKeepAliveHandler,
    WebSocketClosure,
    _log,
    aiohttp,
    asyncio,
    concurrent,
    deque,
    logging,
    namedtuple,
    struct,
    sys,
    threading,
    time,
    traceback,
    utils,
    zlib,
)

__all__ = (
    "DiscordWebSocket",
    "KeepAliveHandler",
    "VoiceKeepAliveHandler",
    "DiscordVoiceWebSocket",
    "ReconnectWebSocket",
)



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.raw_models import (
    BulkMessageDeleteEvent,
    IntegrationDeleteEvent,
    List,
    Member,
    MessageDeleteEvent,
    MessageUpdateEvent,
    PartialEmoji,
    ReactionActionEvent,
    ReactionClearEmojiEvent,
    ReactionClearEvent,
    Snowflake,
    TypedDict,
    TypingEvent,
    _IntegrationDeleteEventOptional,
    _MessageEventOptional,
    _ReactionActionEventOptional,
    _ReactionClearEmojiEventOptional,
    _ReactionClearEventOptional,
    _TypingEventOptional,
)



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.shard import (
    TYPE_CHECKING,
    Any,
    AutoShardedClient,
    AutoShardedConnectionState,
    Callable,
    Client,
    ClientException,
    ConnectionClosed,
    Dict,
    DiscordVoiceWebSocket,
    DiscordWebSocket,
    EventItem,
    EventType,
    ExponentialBackoff,
    GatewayNotFound,
    HTTPException,
    KeepAliveHandler,
    List,
    Optional,
    PrivilegedIntentsRequired,
    ReconnectWebSocket,
    Shard,
    ShardInfo,
    Status,
    Tuple,
    Type,
    TypeVar,
    VoiceKeepAliveHandler,
    _log,
    aiohttp,
    annotations,
    asyncio,
    logging,
)

__all__ = ("AutoShardedClient", "ShardInfo")



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.activity import (
    TYPE_CHECKING,
    Activity,
    ActivityType,
    ActivityTypes,
    Any,
    Asset,
    BaseActivity,
    Colour,
    CustomActivity,
    Dict,
    Game,
    List,
    Optional,
    PartialEmoji,
    Spotify,
    Streaming,
    Union,
    annotations,
    create_activity,
    datetime,
    get_as_snowflake,
    overload,
    try_enum,
)

__all__ = ("BaseActivity", "Activity", "Streaming", "Game", "Spotify", "CustomActivity")



# SPDX-License-Identifier: MIT

from __future__ import annotations

import os
from collections import UserList
from typing import TYPE_CHECKING, List, Optional, Tuple, TypeVar, Union

from ...components import SelectMenu
from ...enums import ComponentType
from ...guild import Guild
from ...member import Member
from ...role import Role
from ...state import ConnectionState
from ...user import User
from ...utils import MISSING
from ..item import Item

__all__ = ("SelectBase",)

if TYPE_CHECKING:
    from typing_extensions import Self

    from ...abc import GuildChannel
    from ...types.components import SelectMenu as SelectMenuPayload
    from ...types.interactions import ComponentInteractionData, ComponentInteractionResolved
    from ..view import View

V = TypeVar("V", bound="View", covariant=True)


class SelectValuesBase(UserList):
    def __init__(self):
        self.data: List[Union[Member, User, Role, GuildChannel]] = []

    @classmethod
    def construct(
        cls,
        values: List[str],
        resolved: ComponentInteractionResolved,
        state: ConnectionState,
        guild: Optional[Guild],
    ):
        instance = cls()
        users = resolved.get("users", {})
        members = resolved.get("members", {})
        roles = resolved.get("roles", {})
        channels = resolved.get("channels", {})
        for value in values:
            if members.get(value) and guild:
                members[value]["user"] = users[value]
                instance.append(Member(data=members[value], state=state, guild=guild))
            elif users.get(value):
                instance.append(User(data=users[value], state=state))
            elif roles.get(value) and guild:
                instance.append(Role(data=roles[value], state=state, guild=guild))
            elif channels.get(value) and guild:
                channel = state.get_channel(int(value))
                instance.append(channel)
        return instance

    @property
    def ids(self) -> List[int]:
        return [o.id for o in self.data]


class SelectBase(Item[V]):
    """Represents a UI select menu without any options.

    This is usually represented as a drop down menu.

    In order to get the selected items that the user has chosen, use :attr:`Select.values`.

    .. warning:: This class is not meant to be instantiated by the user. In order to create a select menu, use :class:`Select` instead.

    .. versionadded:: 2.3

    Parameters
    ----------
    custom_id: :class:`str`
        The ID of the select menu that gets received during an interaction.
        If not given then one is generated for you.
    placeholder: Optional[:class:`str`]
        The placeholder text that is shown if nothing is selected, if any.
    min_values: :class:`int`
        The minimum number of items that must be chosen for this select menu.
        Defaults to 1 and must be between 1 and 25.
    max_values: :class:`int`
        The maximum number of items that must be chosen for this select menu.
        Defaults to 1 and must be between 1 and 25.
    disabled: :class:`bool`
        Whether the select is disabled or not.
    row: Optional[:class:`int`]
        The relative row this select menu belongs to. A Discord component can only have 5
        rows. By default, items are arranged automatically into those 5 rows. If you'd
        like to control the relative positioning of the row then passing an index is advised.
        For example, row=1 will show up before row=2. Defaults to ``None``, which is automatic
        ordering. The row number must be between 0 and 4 (i.e. zero indexed).
    """

    __item_repr_attributes__: Tuple[str, ...] = (
        "placeholder",
        "min_values",
        "max_values",
        "options",
        "disabled",
    )

    def __init__(
        self,
        *,
        custom_id: str = MISSING,
        placeholder: Optional[str] = None,
        min_values: int = 1,
        max_values: int = 1,
        disabled: bool = False,
        row: Optional[int] = None,
    ) -> None:
        super().__init__()
        self._selected_values: List[str] = []
        self._provided_custom_id = custom_id is not MISSING
        custom_id = os.urandom(16).hex() if custom_id is MISSING else custom_id
        self._underlying = SelectMenu._raw_construct(
            custom_id=custom_id,
            type=ComponentType.select,
            placeholder=placeholder,
            min_values=min_values,
            max_values=max_values,
            disabled=disabled,
        )
        self.row = row

    @property
    def custom_id(self) -> str:
        """:class:`str`: The ID of the select menu that gets received during an interaction."""
        return self._underlying.custom_id

    @custom_id.setter
    def custom_id(self, value: str) -> None:
        if not isinstance(value, str):
            raise TypeError("custom_id must be None or str")

        self._underlying.custom_id = value

    @property
    def placeholder(self) -> Optional[str]:
        """Optional[:class:`str`]: The placeholder text that is shown if nothing is selected, if any."""
        return self._underlying.placeholder

    @placeholder.setter
    def placeholder(self, value: Optional[str]):
        if value is not None and not isinstance(value, str):
            raise TypeError("placeholder must be None or str")

        self._underlying.placeholder = value

    @property
    def min_values(self) -> int:
        """:class:`int`: The minimum number of items that must be chosen for this select menu."""
        return self._underlying.min_values

    @min_values.setter
    def min_values(self, value: int) -> None:
        self._underlying.min_values = int(value)

    @property
    def max_values(self) -> int:
        """:class:`int`: The maximum number of items that must be chosen for this select menu."""
        return self._underlying.max_values

    @max_values.setter
    def max_values(self, value: int) -> None:
        self._underlying.max_values = int(value)

    @property
    def disabled(self) -> bool:
        """:class:`bool`: Whether the select is disabled or not."""
        return self._underlying.disabled

    @disabled.setter
    def disabled(self, value: bool) -> None:
        self._underlying.disabled = bool(value)

    @property
    def values(self) -> List[str]:
        """List[:class:`str`]: A list of values that have been selected by the user."""
        return self._selected_values

    @property
    def width(self) -> int:
        return 5

    def to_component_dict(self) -> SelectMenuPayload:
        return self._underlying.to_dict()

    def refresh_component(self, component: SelectMenu) -> None:
        self._underlying = component

    def refresh_state(
        self, data: ComponentInteractionData, state: ConnectionState, guild: Optional[Guild]
    ) -> None:
        self._selected_values = data.get("values", [])

    @classmethod
    def from_component(cls, component: SelectMenu) -> Self:
        return cls(
            custom_id=component.custom_id,
            placeholder=component.placeholder,
            min_values=component.min_values,
            max_values=component.max_values,
            disabled=component.disabled,
            row=None,
        )

    @property
    def type(self) -> ComponentType:
        return self._underlying.type

    def is_dispatchable(self) -> bool:
        return True



# SPDX-License-Identifier: MIT

from __future__ import annotations

from typing import TYPE_CHECKING, List, Optional, Union

from .appinfo import PartialAppInfo
from .asset import Asset
from .enums import ChannelType, InviteTarget, VerificationLevel, try_enum
from .mixins import Hashable
from .object import Object
from .utils import get_as_snowflake, parse_time, snowflake_time

__all__ = (
    "PartialInviteChannel",
    "PartialInviteGuild",
    "Invite",
)

if TYPE_CHECKING:
    from typing_extensions import Self

    from .abc import GuildChannel
    from .guild import Guild
    from .state import ConnectionState
    from .types.channel import PartialChannel as InviteChannelPayload
    from .types.invite import (
        GatewayInvite as GatewayInvitePayload,
        Invite as InvitePayload,
        InviteGuild as InviteGuildPayload,
        VanityInvite as VanityInvitePayload,
    )
    from .user import User

    InviteGuildType = Union[Guild, "PartialInviteGuild", Object]
    InviteChannelType = Union[GuildChannel, "PartialInviteChannel", Object]

    import datetime


class PartialInviteChannel:
    """Represents a "partial" invite channel.

    This model will be given when the user is not part of the
    guild the :class:`Invite` resolves to.

    .. container:: operations

        .. describe:: x == y

            Checks if two partial channels are the same.

        .. describe:: x != y

            Checks if two partial channels are not the same.

        .. describe:: hash(x)

            Return the partial channel's hash.

        .. describe:: str(x)

            Returns the partial channel's name.

    Attributes
    ----------
    name: :class:`str`
        The partial channel's name.
    id: :class:`int`
        The partial channel's ID.
    type: :class:`ChannelType`
        The partial channel's type.
    """

    __slots__ = ("id", "name", "type")

    def __init__(self, data: InviteChannelPayload):
        self.id: int = int(data["id"])
        self.name: str = data["name"]
        self.type: ChannelType = try_enum(ChannelType, data["type"])

    def __str__(self) -> str:
        return self.name

    def __repr__(self) -> str:
        return f"<PartialInviteChannel id={self.id} name={self.name} type={self.type!r}>"

    @property
    def mention(self) -> str:
        """:class:`str`: The string that allows you to mention the channel."""
        return f"<#{self.id}>"

    @property
    def created_at(self) -> datetime.datetime:
        """:class:`datetime.datetime`: Returns the channel's creation time in UTC."""
        return snowflake_time(self.id)


class PartialInviteGuild:
    """Represents a "partial" invite guild.

    This model will be given when the user is not part of the
    guild the :class:`Invite` resolves to.

    .. container:: operations

        .. describe:: x == y

            Checks if two partial guilds are the same.

        .. describe:: x != y

            Checks if two partial guilds are not the same.

        .. describe:: hash(x)

            Return the partial guild's hash.

        .. describe:: str(x)

            Returns the partial guild's name.

    Attributes
    ----------
    name: :class:`str`
        The partial guild's name.
    id: :class:`int`
        The partial guild's ID.
    verification_level: :class:`VerificationLevel`
        The partial guild's verification level.
    features: List[:class:`str`]
        A list of features the guild has. See :attr:`Guild.features` for more information.
    description: Optional[:class:`str`]
        The partial guild's description.
    """

    __slots__ = (
        "_state",
        "features",
        "_icon",
        "_banner",
        "id",
        "name",
        "_splash",
        "verification_level",
        "description",
    )

    def __init__(self, state: ConnectionState, data: InviteGuildPayload, id: int):
        self._state: ConnectionState = state
        self.id: int = id
        self.name: str = data["name"]
        self.features: List[str] = data.get("features", [])
        self._icon: Optional[str] = data.get("icon")
        self._banner: Optional[str] = data.get("banner")
        self._splash: Optional[str] = data.get("splash")
        self.verification_level: VerificationLevel = try_enum(
            VerificationLevel, data.get("verification_level")
        )
        self.description: Optional[str] = data.get("description")

    def __str__(self) -> str:
        return self.name

    def __repr__(self) -> str:
        return (
            f"<{self.__class__.__name__} id={self.id} name={self.name!r} features={self.features} "
            f"description={self.description!r}>"
        )

    @property
    def created_at(self) -> datetime.datetime:
        """:class:`datetime.datetime`: Returns the guild's creation time in UTC."""
        return snowflake_time(self.id)

    @property
    def icon(self) -> Optional[Asset]:
        """Optional[:class:`Asset`]: Returns the guild's icon asset, if available."""
        if self._icon is None:
            return None
        return Asset._from_guild_icon(self._state, self.id, self._icon)

    @property
    def banner(self) -> Optional[Asset]:
        """Optional[:class:`Asset`]: Returns the guild's banner asset, if available."""
        if self._banner is None:
            return None
        return Asset._from_guild_image(self._state, self.id, self._banner, path="banners")

    @property
    def splash(self) -> Optional[Asset]:
        """Optional[:class:`Asset`]: Returns the guild's invite splash asset, if available."""
        if self._splash is None:
            return None
        return Asset._from_guild_image(self._state, self.id, self._splash, path="splashes")


class Invite(Hashable):
    r"""Represents a Discord :class:`Guild` or :class:`abc.GuildChannel` invite.

    Depending on the way this object was created, some of the attributes can
    have a value of ``None``.

    .. container:: operations

        .. describe:: x == y

            Checks if two invites are equal.

        .. describe:: x != y

            Checks if two invites are not equal.

        .. describe:: hash(x)

            Returns the invite hash.

        .. describe:: str(x)

            Returns the invite URL.

    The following table illustrates what methods will obtain the attributes:

    +------------------------------------+--------------------------------------------------------------+
    |             Attribute              |                          Method                              |
    +====================================+==============================================================+
    | :attr:`max_age`                    | :meth:`abc.GuildChannel.invites`\, :meth:`Guild.invites`     |
    +------------------------------------+--------------------------------------------------------------+
    | :attr:`max_uses`                   | :meth:`abc.GuildChannel.invites`\, :meth:`Guild.invites`     |
    +------------------------------------+--------------------------------------------------------------+
    | :attr:`created_at`                 | :meth:`abc.GuildChannel.invites`\, :meth:`Guild.invites`     |
    +------------------------------------+--------------------------------------------------------------+
    | :attr:`temporary`                  | :meth:`abc.GuildChannel.invites`\, :meth:`Guild.invites`     |
    +------------------------------------+--------------------------------------------------------------+
    | :attr:`uses`                       | :meth:`abc.GuildChannel.invites`\, :meth:`Guild.invites`     |
    +------------------------------------+--------------------------------------------------------------+
    | :attr:`approximate_member_count`   | :meth:`Client.fetch_invite` with ``with_counts`` enabled     |
    +------------------------------------+--------------------------------------------------------------+
    | :attr:`approximate_presence_count` | :meth:`Client.fetch_invite` with ``with_counts`` enabled     |
    +------------------------------------+--------------------------------------------------------------+
    | :attr:`expires_at`                 | :meth:`Client.fetch_invite` with ``with_expiration`` enabled |
    +------------------------------------+--------------------------------------------------------------+

    If it's not in the table above then it is available by all methods.

    Attributes
    ----------
    max_age: :class:`int`
        How long before the invite expires in seconds.
        A value of ``0`` indicates that it doesn't expire.
    code: :class:`str`
        The URL fragment used for the invite.
    guild: Optional[Union[:class:`Guild`, :class:`Object`, :class:`PartialInviteGuild`]]
        The guild the invite is for. Can be ``None`` if it's from a group direct message.
    revoked: :class:`bool`
        Indicates if the invite has been revoked.
    created_at: :class:`datetime.datetime`
        An aware UTC datetime object denoting the time the invite was created.
    temporary: :class:`bool`
        Indicates that the invite grants temporary membership.
        If ``True``, members who joined via this invite will be kicked upon disconnect.
    uses: :class:`int`
        How many times the invite has been used.
    max_uses: :class:`int`
        How many times the invite can be used.
        A value of ``0`` indicates that it has unlimited uses.
    inviter: Optional[:class:`User`]
        The user who created the invite.
    approximate_member_count: Optional[:class:`int`]
        The approximate number of members in the guild.
    approximate_presence_count: Optional[:class:`int`]
        The approximate number of members currently active in the guild.
        This includes idle, dnd, online, and invisible members. Offline members are excluded.
    expires_at: Optional[:class:`datetime.datetime`]
        The expiration date of the invite. If the value is ``None`` when received through
        :meth:`Client.fetch_invite` with ``with_expiration`` enabled, the invite will never expire.

        .. versionadded:: 2.0

    channel: Union[:class:`abc.GuildChannel`, :class:`Object`, :class:`PartialInviteChannel`]
        The channel the invite is for.
    target_type: :class:`InviteTarget`
        The type of target for the voice channel invite.

        .. versionadded:: 2.0

    target_user: Optional[:class:`User`]
        The user whose stream to display for this invite, if any.

        .. versionadded:: 2.0

    target_application: Optional[:class:`PartialAppInfo`]
        The embedded application the invite targets, if any.

        .. versionadded:: 2.0
    """

    __slots__ = (
        "max_age",
        "code",
        "guild",
        "revoked",
        "created_at",
        "uses",
        "temporary",
        "max_uses",
        "inviter",
        "channel",
        "target_user",
        "target_type",
        "_state",
        "approximate_member_count",
        "approximate_presence_count",
        "target_application",
        "expires_at",
    )

    BASE = "https://discord.gg"

    def __init__(
        self,
        *,
        state: ConnectionState,
        data: InvitePayload | VanityInvitePayload,
        guild: Optional[Union[PartialInviteGuild, Guild]] = None,
        channel: Optional[Union[PartialInviteChannel, GuildChannel]] = None,
    ):
        self._state: ConnectionState = state
        self.max_age: Optional[int] = data.get("max_age")
        self.code: Optional[str] = data.get("code")
        self.guild: Optional[InviteGuildType] = self._resolve_guild(data.get("guild"), guild)
        self.revoked: Optional[bool] = data.get("revoked")
        self.created_at: Optional[datetime.datetime] = parse_time(data.get("created_at"))
        self.temporary: Optional[bool] = data.get("temporary")
        self.uses: Optional[int] = data.get("uses")
        self.max_uses: Optional[int] = data.get("max_uses")
        self.approximate_presence_count: Optional[int] = data.get("approximate_presence_count")
        self.approximate_member_count: Optional[int] = data.get("approximate_member_count")

        expires_at = data.get("expires_at", None)
        self.expires_at: Optional[datetime.datetime] = (
            parse_time(expires_at) if expires_at else None
        )

        inviter_data = data.get("inviter")
        self.inviter: Optional[User] = (
            None if inviter_data is None else self._state.create_user(inviter_data)
        )

        self.channel: Optional[InviteChannelType] = self._resolve_channel(
            data.get("channel"), channel
        )

        target_user_data = data.get("target_user")
        self.target_user: Optional[User] = (
            None if target_user_data is None else self._state.create_user(target_user_data)
        )

        self.target_type: InviteTarget = try_enum(InviteTarget, data.get("target_type", 0))

        application = data.get("target_application")
        self.target_application: Optional[PartialAppInfo] = (
            PartialAppInfo(data=application, state=state) if application else None
        )

    @classmethod
    def from_incomplete(cls, *, state: ConnectionState, data: InvitePayload) -> Self:
        guild: Optional[Union[Guild, PartialInviteGuild]]
        try:
            guild_data = data["guild"]
        except KeyError:
            # If we're here, then this is a group DM
            guild = None
        else:
            guild_id = int(guild_data["id"])
            guild = state._get_guild(guild_id)
            if guild is None:
                # If it's not cached, then it has to be a partial guild
                guild = PartialInviteGuild(state, guild_data, guild_id)

        # As far as I know, invites always need a channel
        # So this should never raise.
        channel: Union[PartialInviteChannel, GuildChannel] = PartialInviteChannel(data["channel"])
        if guild is not None and not isinstance(guild, PartialInviteGuild):
            # Upgrade the partial data if applicable
            channel = guild.get_channel(channel.id) or channel

        return cls(state=state, data=data, guild=guild, channel=channel)

    @classmethod
    def from_gateway(cls, *, state: ConnectionState, data: GatewayInvitePayload) -> Self:
        guild_id: Optional[int] = get_as_snowflake(data, "guild_id")
        guild: Optional[Union[Guild, Object]] = state._get_guild(guild_id)
        channel_id = int(data["channel_id"])
        if guild is not None:
            channel = guild.get_channel(channel_id) or Object(id=channel_id)
        else:
            guild = Object(id=guild_id) if guild_id is not None else None
            channel = Object(id=channel_id)

        return cls(state=state, data=data, guild=guild, channel=channel)  # type: ignore

    def _resolve_guild(
        self,
        data: Optional[InviteGuildPayload],
        guild: Optional[Union[Guild, PartialInviteGuild]] = None,
    ) -> Optional[InviteGuildType]:
        if guild is not None:
            return guild

        if data is None:
            return None

        guild_id = int(data["id"])
        return PartialInviteGuild(self._state, data, guild_id)

    def _resolve_channel(
        self,
        data: Optional[InviteChannelPayload],
        channel: Optional[Union[PartialInviteChannel, GuildChannel]] = None,
    ) -> Optional[InviteChannelType]:
        if channel is not None:
            return channel

        if data is None:
            return None

        return PartialInviteChannel(data)

    def __str__(self) -> str:
        return self.url

    def __repr__(self) -> str:
        return (
            f"<Invite code={self.code!r} guild={self.guild!r} "
            f"online={self.approximate_presence_count} "
            f"members={self.approximate_member_count}>"
        )

    def __hash__(self) -> int:
        return hash(self.code)

    @property
    def id(self) -> Optional[str]:
        """Optional[:class:`str`]: Returns the proper code portion of the invite.

        .. note::

            This may be ``None`` if it is the vanity invite the code is not set.
        """
        return self.code

    @property
    def url(self) -> str:
        """:class:`str`: A property that retrieves the invite URL.

        .. note::

            This may be an empty string if it is the vanity URL and the code is not set
        """
        return self.BASE + "/" + self.code if self.code else ""

    async def delete(self, *, reason: Optional[str] = None):
        """|coro|

        Revokes the instant invite.

        You must have the :attr:`~Permissions.manage_channels` permission to do this.

        Parameters
        ----------
        reason: Optional[:class:`str`]
            The reason for deleting this invite. Shows up on the audit log.

        Raises
        ------
        Forbidden
            You do not have permissions to revoke invites.
        NotFound
            The invite is invalid or expired.
        HTTPException
            Revoking the invite failed.
        """

        if not self.code:
            return

        await self._state.http.delete_invite(self.code, reason=reason)



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.embed import (
    Embed,
    EmbedAuthor,
    EmbedField,
    EmbedFooter,
    EmbedImage,
    EmbedProvider,
    EmbedThumbnail,
    EmbedType,
    EmbedVideo,
    List,
    Literal,
    TypedDict,
    _EmbedFieldOptional,
    _EmbedFooterOptional,
)



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.health_check import (
    DistributionNotFound,
    DistributionWarning,
    get_distribution,
    incompatible_libraries,
    library,
    warn,
)

__all__ = "incompatible_libraries"



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.widget import (
    TYPE_CHECKING,
    Activity,
    Any,
    BaseActivity,
    BaseUser,
    Invite,
    List,
    Optional,
    Spotify,
    Status,
    Union,
    Widget,
    WidgetChannel,
    WidgetMember,
    annotations,
    create_activity,
    get_as_snowflake,
    resolve_invite,
    snowflake_time,
    try_enum,
)

__all__ = ("WidgetChannel", "WidgetMember", "Widget")



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.webhook.sync import (
    MISSING,
    TYPE_CHECKING,
    Any,
    BaseWebhook,
    DeferredLock,
    Dict,
    DiscordServerError,
    Forbidden,
    HTTPException,
    InvalidArgument,
    List,
    Literal,
    Message,
    NotFound,
    Optional,
    PartialMessageable,
    Route,
    SyncWebhook,
    SyncWebhookMessage,
    Tuple,
    Type,
    TypeVar,
    Union,
    WebhookAdapter,
    _context,
    _get_webhook_adapter,
    _log,
    _WebhookContext,
    _WebhookState,
    annotations,
    handle_message_parameters,
    json,
    logging,
    overload,
    re,
    threading,
    time,
    urlquote,
    utils,
)

__all__ = ("SyncWebhook", "SyncWebhookMessage")



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.player import (
    AT,
    CREATE_NO_WINDOW,
    FT,
    IO,
    MISSING,
    TYPE_CHECKING,
    Any,
    AudioPlayer,
    AudioSource,
    Callable,
    ClientException,
    FFmpegAudio,
    FFmpegOpusAudio,
    FFmpegPCMAudio,
    Generic,
    OggStream,
    Optional,
    OpusEncoder,
    PCMAudio,
    PCMVolumeTransformer,
    Tuple,
    Type,
    TypeVar,
    Union,
    _log,
    annotations,
    asyncio,
    audioop,
    io,
    json,
    logging,
    re,
    shlex,
    subprocess,
    sys,
    threading,
    time,
    traceback,
)

__all__ = (
    "AudioSource",
    "PCMAudio",
    "FFmpegAudio",
    "FFmpegPCMAudio",
    "FFmpegOpusAudio",
    "PCMVolumeTransformer",
)



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.message import (
    MISSING,
    TYPE_CHECKING,
    Any,
    Attachment,
    Callable,
    ChannelType,
    ClassVar,
    DeletedReferencedMessage,
    Dict,
    Embed,
    Emoji,
    File,
    Guild,
    Hashable,
    HTTPException,
    InvalidArgument,
    List,
    Member,
    Message,
    MessageFlags,
    MessageReference,
    MessageType,
    Optional,
    PartialEmoji,
    PartialMessage,
    PathLike,
    Reaction,
    StickerItem,
    Thread,
    Tuple,
    Type,
    TypeVar,
    Union,
    _component_factory,
    annotations,
    asyncio,
    convert_emoji_reaction,
    datetime,
    escape_mentions,
    flatten_handlers,
    io,
    overload,
    re,
    try_enum,
    utils,
)

__all__ = (
    "Attachment",
    "Message",
    "PartialMessage",
    "MessageReference",
    "DeletedReferencedMessage",
)



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.guild import (
    MISSING,
    TYPE_CHECKING,
    Any,
    Asset,
    AuditLogAction,
    AuditLogIterator,
    BanEntry,
    CategoryChannel,
    ChannelType,
    ClassVar,
    ClientException,
    Colour,
    ContentFilter,
    Dict,
    DMChannel,
    Emoji,
    File,
    GroupChannel,
    Guild,
    GuildSticker,
    Hashable,
    Integration,
    InvalidArgument,
    InvalidData,
    Invite,
    List,
    Literal,
    Member,
    MemberIterator,
    NamedTuple,
    NotificationLevel,
    NSFWLevel,
    Optional,
    PartialMessageable,
    PermissionOverwrite,
    Role,
    Sequence,
    Set,
    StageChannel,
    StageInstance,
    SystemChannelFlags,
    TextChannel,
    Thread,
    ThreadMember,
    Tuple,
    Union,
    User,
    VerificationLevel,
    VideoQualityMode,
    VoiceChannel,
    VoiceRegion,
    VoiceState,
    Widget,
    _guild_channel_factory,
    _GuildLimit,
    _integration_factory,
    _threaded_guild_channel_factory,
    abc,
    annotations,
    copy,
    overload,
    try_enum,
    unicodedata,
    utils,
)

__all__ = "Guild"



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.role import (
    MISSING,
    TYPE_CHECKING,
    Any,
    Colour,
    Dict,
    Hashable,
    InvalidArgument,
    List,
    Optional,
    Permissions,
    R,
    Role,
    RoleTags,
    TypeVar,
    Union,
    annotations,
    get_as_snowflake,
    overload,
    snowflake_time,
)

__all__ = ("RoleTags", "Role")



# SPDX-License-Identifier: MIT

from __future__ import annotations

from typing import TypedDict

from typing_extensions import NotRequired

from .snowflake import Snowflake


class Role(TypedDict):
    id: Snowflake
    name: str
    color: int
    hoist: bool
    position: int
    permissions: str
    managed: bool
    mentionable: bool
    tags: NotRequired[RoleTags]
    unicode_emoji: NotRequired[str]
    icon: NotRequired[str]


class RoleTags(TypedDict, total=False):
    bot_id: Snowflake
    integration_id: Snowflake
    premium_subscriber: None



# SPDX-License-Identifier: MIT

from __future__ import annotations

from typing import TYPE_CHECKING, List, Optional

from .enums import (
    AutoModerationActionType,
    AutoModerationEventType,
    AutoModerationTriggerType,
    KeywordPresetType,
    try_enum,
)
from .errors import InvalidArgument
from .mixins import Hashable
from .object import Object
from .utils import MISSING, get_as_snowflake

if TYPE_CHECKING:
    from .abc import GuildChannel, Snowflake
    from .guild import Guild
    from .member import Member
    from .message import Message
    from .role import Role
    from .state import ConnectionState
    from .types.auto_moderation import (
        AutoModerationAction as AutoModerationActionPayload,
        AutoModerationActionExecution as ActionExecutionPayload,
        AutoModerationActionMetadata as ActionMetadataPayload,
        AutoModerationRule as AutoModerationRulePayload,
        AutoModerationRuleModify,
        AutoModerationTriggerMetadata as TriggerMetadataPayload,
    )

__all__ = (
    "AutoModerationTriggerMetadata",
    "AutoModerationActionMetadata",
    "AutoModerationAction",
    "AutoModerationRule",
    "AutoModerationActionExecution",
)


class AutoModerationTriggerMetadata:
    """Represents data about an auto moderation trigger rule.

    .. versionadded:: 2.1

    Attributes
    ----------
    keyword_filter: List[:class:`str`]
        A list of substrings which will be searched for in content.

        .. note::

            This is ``None`` and cannot be provided if the trigger type of the rule is not
            :attr:`AutoModerationTriggerType.keyword`.
    presets: List[:class:`KeywordPresetType`]
        A list of Discord pre-defined wordsets which will be searched for in content.

        .. note::

            This is ``None`` and cannot be provided if the trigger type of the rule is not
            :attr:`AutoModerationTriggerType.keyword_preset`.
    allow_list: List[:class:`str`]
        A list of exempt strings that will not trigger the preset type.

        .. note::

            This is ``None`` and cannot be provided if the trigger type of the rule is not
            :attr:`AutoModerationTriggerType.keyword_preset`.

        .. warning::

            Wildcard syntax (`*`) is not supported here.
    mention_total_limit: Optional[:class:`int`]
        The maximum amount of mentions allowed in a messsage.

        .. versionadded:: 2.3

        .. note::

            This is ``None`` and cannot be provided if the trigger type of this rule is not :attr:`AutoModerationTriggerType.mention_spam`.
    """

    __slots__ = ("keyword_filter", "presets", "allow_list", "mention_total_limit")

    def __init__(
        self,
        *,
        keyword_filter: Optional[List[str]] = None,
        presets: Optional[List[KeywordPresetType]] = None,
        allow_list: Optional[List[str]] = None,
        mention_total_limit: Optional[int] = None,
    ) -> None:
        self.keyword_filter: Optional[List[str]] = keyword_filter
        self.presets: Optional[List[KeywordPresetType]] = presets
        self.allow_list: Optional[List[str]] = allow_list
        self.mention_total_limit: Optional[int] = mention_total_limit

    @classmethod
    def from_data(cls, data: TriggerMetadataPayload):
        keyword_filter = data.get("keyword_filter")
        presets = (
            [try_enum(KeywordPresetType, preset) for preset in data["presets"]]
            if "presets" in data
            else None
        )
        allow_list = data.get("allow_list")
        mention_total_limit = data.get("mention_total_limit")

        return cls(
            keyword_filter=keyword_filter,
            presets=presets,
            allow_list=allow_list,
            mention_total_limit=mention_total_limit,
        )

    @property
    def payload(self) -> TriggerMetadataPayload:
        payload: TriggerMetadataPayload = {}

        if self.keyword_filter is not None:
            payload["keyword_filter"] = self.keyword_filter

        if self.presets is not None:
            payload["presets"] = [enum.value for enum in self.presets]

        if self.allow_list is not None:
            payload["allow_list"] = self.allow_list

        if self.mention_total_limit is not None:
            payload["mention_total_limit"] = self.mention_total_limit

        return payload


class AutoModerationActionMetadata:
    """Represents additional data that is used when an action is executed.

    .. versionadded:: 2.1

    Attributes
    ----------
    channel: Optional[:class:`abc.Snowflake`]
        The channel to which message content should be logged.

        .. note::

            This is ``None`` and cannot be provided if the action type of the rule is not
            :attr:`AutoModerationActionType.send_alert_message`.
    duration_seconds: Optional[:class:`int`]
        The duration of the timeout in seconds.

        .. note::

            This is ``None`` and cannot be provided if the action type of the rule is not
            :attr:`AutoModerationActionType.timeout`.

        .. note::

            The maximum value that can be used is ``2419200`` seconds (4 weeks)
    """

    __slots__ = ("channel_id", "duration_seconds")

    def __init__(
        self, *, channel: Optional[Snowflake] = None, duration_seconds: Optional[int] = None
    ):
        self.channel_id: Optional[int] = channel.id if channel is not None else None
        self.duration_seconds: Optional[int] = duration_seconds

    @classmethod
    def from_data(cls, data: ActionMetadataPayload):
        channel_id = get_as_snowflake(data, "channel_id")
        channel = Object(id=channel_id) if channel_id is not None else None
        duration_seconds = data.get("duration_seconds")

        return cls(channel=channel, duration_seconds=duration_seconds)

    @property
    def payload(self) -> ActionMetadataPayload:
        payload: ActionMetadataPayload = {}

        if self.channel_id is not None:
            payload["channel_id"] = self.channel_id

        if self.duration_seconds is not None:
            payload["duration_seconds"] = self.duration_seconds

        return payload


class AutoModerationAction:
    """Represents an auto moderation action that will execute whenever a rule is triggered.

    .. versionadded:: 2.1

    Parameters
    ----------
    type: :class:`AutoModerationActionType`
        The type to use for this action.
    metadata: :class:`AutoModerationActionMetadata`
        The additional data to use during execution of this action.

    Attributes
    ----------
    type: :class:`AutoModerationActionType`
        The type of this action.
    metadata: :class:`AutoModerationActionMetadata`
        The additional metadata needed during execution for this specific action type.
    """

    __slots__ = ("type", "metadata")

    def __init__(
        self,
        *,
        type: AutoModerationActionType,
        metadata: Optional[AutoModerationActionMetadata] = None,
    ) -> None:
        self.type: AutoModerationActionType = type
        self.metadata: Optional[AutoModerationActionMetadata] = metadata

    @classmethod
    def from_data(cls, data: AutoModerationActionPayload) -> AutoModerationAction:
        type = try_enum(AutoModerationActionType, data["type"])
        metadata = AutoModerationActionMetadata.from_data(data.get("metadata", {}))
        return cls(type=type, metadata=metadata)

    @property
    def payload(self) -> AutoModerationActionPayload:
        data: AutoModerationActionPayload = {
            "type": self.type.value,
        }

        if self.metadata is not None:
            data["metadata"] = self.metadata.payload

        return data


class AutoModerationRule(Hashable):
    """Represents a Discord auto moderation rule.

    .. versionadded:: 2.1

    .. container:: operations

        .. describe:: x == y

            Checks if two rules are equal.

        .. describe:: x != y

            Checks if two rules are not equal.

        .. describe:: hash(x)

            Returns the rules's hash.

        .. describe:: str(x)

            Returns the rules's name.

    Attributes
    ----------
    id: :class:`int`
        The rule's unique ID.
    guild_id: :class:`int`
        The guild's unique ID which this rule belongs to.
    guild: Optional[:class:`Guild`]
        The guild which this rule belongs to, if found in cache.
    name: :class:`str`
        The rule's name.
    creator_id: :class:`int`
        The user's unique ID which first created this rule.
    creator: Optional[:class:`Member`]
        The member which first created this rule, if found in cache.
    event_type: :class:`AutoModerationEventType`
        The event context in which this rule is checked.
    trigger_type: :class:`AutoModerationTriggerType`
        The type of content that can trigger this rule.
    trigger_metadata: :class:`AutoModerationTriggerMetadata`
        Additional data that is used when checking if this rule is triggered.
    actions: List[:class:`AutoModerationAction`]
        The actions which will execute when the rule is triggered.
    enabled: :class:`bool`
        Whether the rule is enabled.
    exempt_role_ids: List[:class:`int`]
        The role ids that should not be affected by the rule.
    exempt_channel_ids: List[:class:`int`]
        The channel ids that should not be affected by the rule.
    exempt_roles: List[:class:`Role`]
        The roles that should not be affected by the rule, if found in cache.
    exempt_channels: List[:class:`abc.GuildChannel`]
        The channels that should not be affected by the rule, if found in cache.
    """

    __slots__ = (
        "_state",
        "id",
        "guild_id",
        "guild",
        "name",
        "creator_id",
        "creator",
        "event_type",
        "trigger_type",
        "trigger_metadata",
        "actions",
        "enabled",
        "exempt_role_ids",
        "exempt_channel_ids",
        "exempt_roles",
        "exempt_channels",
    )

    def __init__(self, *, data: AutoModerationRulePayload, state: ConnectionState):
        self._state = state
        self.id: int = int(data["id"])
        self.guild_id: int = int(data["guild_id"])
        self.guild: Optional[Guild] = state._get_guild(self.guild_id)
        self.name: str = data["name"]
        self.creator_id: int = int(data["creator_id"])
        self.creator: Optional[Member] = (
            self.guild.get_member(self.creator_id) if self.guild is not None else None
        )
        self.event_type: AutoModerationEventType = try_enum(
            AutoModerationEventType, data["event_type"]
        )
        self.trigger_type: AutoModerationTriggerType = try_enum(
            AutoModerationTriggerType, data["trigger_type"]
        )
        self.trigger_metadata: AutoModerationTriggerMetadata = (
            AutoModerationTriggerMetadata.from_data(data["trigger_metadata"])
        )
        self.actions: List[AutoModerationAction] = [
            AutoModerationAction.from_data(data=action) for action in data["actions"]
        ]
        self.enabled: bool = data["enabled"]
        self.exempt_role_ids: List[int] = [int(role_id) for role_id in data["exempt_roles"]]
        self.exempt_channel_ids: List[int] = [
            int(channel_id) for channel_id in data["exempt_channels"]
        ]
        self.exempt_roles: List[Optional[Role]] = (
            [self.guild.get_role(role_id) for role_id in self.exempt_role_ids]
            if self.guild is not None
            else []
        )
        self.exempt_channels: List[Optional[GuildChannel]] = (
            [self.guild.get_channel(channel_id) for channel_id in self.exempt_channel_ids]
            if self.guild is not None
            else []
        )

    def __str__(self) -> str:
        return self.name

    async def delete(self, *, reason: Optional[str] = None) -> None:
        """|coro|

        Delete the current rule from its guild.

        Requires the :attr:`~Permissions.manage_guild` permission.

        Parameters
        ----------
        reason: Optional[:class:`str`]
            The reason for deleting this rule. Shows up on the audit log.

        Raises
        ------
        Forbidden
            You do not have permission to delete this rule.
        HTTPException
            Deleting the rule failed.
        """

        await self._state.http.delete_auto_moderation_rule(self.guild_id, self.id, reason=reason)

    async def edit(
        self,
        *,
        name: str = MISSING,
        event_type: AutoModerationEventType = MISSING,
        trigger_metadata: AutoModerationTriggerMetadata = MISSING,
        actions: List[AutoModerationAction] = MISSING,
        enabled: bool = MISSING,
        exempt_roles: List[Snowflake] = MISSING,
        exempt_channels: List[Snowflake] = MISSING,
        reason: Optional[str] = None,
    ) -> AutoModerationRule:
        """Modify the current auto moderation rule.

        Requires the :attr:`~Permissions.manage_guild` permission.

        Parameters
        ----------
        name: :class:`str`
            The new name for this rule.
        event_type: :class:`AutoModerationEventType`
            The new event context in which this rule is checked.
        trigger_metadata: :class:`AutoModerationTriggerMetadata`
            The new additional data that is used when checking if this rule is triggered.
        actions: List[:class:`AutoModerationAction`]
            The new actions which will execute when the rule is triggered.
        enabled: :class:`bool`
            Whether the rule is enabled.
        exempt_roles: List[:class:`abc.Snowflake`]
            The new roles that should not be affected by the rule.
        exempt_channels: List[:class:`abc.Snowflake`]
            The new channels that should not be affected by the rule.
        reason: Optional[:class:`str`]
            The reason for editing this rule. Shows in the audit log.

        Raises
        ------
        Forbidden
            You do not have permission to edit this rule.
        HTTPException
            Editing the rule failed.
        InvalidArgument
            An incorrect type was passed.

        Returns
        -------
        :class:`AutoModerationRule`
            The newly edited auto moderation rule.
        """

        payload: AutoModerationRuleModify = {}

        if name is not MISSING:
            payload["name"] = name

        if event_type is not MISSING:
            if not isinstance(event_type, AutoModerationEventType):
                raise InvalidArgument("event_type must be of type AutoModerationEventType")

            payload["event_type"] = event_type.value

        if trigger_metadata is not MISSING:
            if not isinstance(trigger_metadata, AutoModerationTriggerMetadata):
                raise InvalidArgument(
                    "trigger_metadata must be of type AutoModerationTriggerMetadata"
                )

            payload["trigger_metadata"] = trigger_metadata.payload

        if actions is not MISSING:
            payload["actions"] = [action.payload for action in actions]

        if enabled is not MISSING:
            payload["enabled"] = enabled

        if exempt_roles is not MISSING:
            payload["exempt_roles"] = [str(role.id) for role in exempt_roles]

        if exempt_channels is not MISSING:
            payload["exempt_channels"] = [str(channel.id) for channel in exempt_channels]

        data = await self._state.http.modify_auto_moderation_rule(
            self.guild_id, self.id, data=payload, reason=reason
        )
        return AutoModerationRule(data=data, state=self._state)


class AutoModerationActionExecution:
    """Represents the execution of an auto moderation action

    .. versionadded:: 2.1

    Attributes
    ----------
    guild_id: :class:`int`
        The guild ID where this action was executed.
    guild: Optional[:class:`Guild`]
        The guild where this action was executed, if it was found in cache.
    channel_id: Optional[:class:`int`]
        The channel ID where this action was executed, if applicable.
    channel: Optional[:class:`abc.GuildChannel`]
        The channel where this action was executed, if applicable and found in cache.
    message_id: Optional[:class:`int`]
        The message ID that executed this action, if it was not blocked.
    message: Optional[:class:`Message`]
        The message that executed this action, if it was not blocked and found in cache.
    alert_system_message_id: Optional[:class:`int`]
        The ID of the system alert message, if sent.
    alert_system_message: Optional[:class:`Message`]
        The system alert message, if sent and found in cache.
    action: :class:`AutoModerationAction`
        The action that was executed.
    rule_id: :class:`int`
        The id of the rule that was executed.
    rule_trigger_type: :class:`AutoModerationTriggerType`
        The type of rule that was executed.
    member_id: :class:`int`
        The ID of the user that triggered this action.
    member: Optional[:class:`Member`]
        The member that triggered this action, if found in cache.
    content: :class:`str`
        The content the user sent in the message

        .. note::

            This requires :attr:`Intents.message_content` to not be empty.
    matched_keyword: Optional[:class:`str`]
        The keyword configured in the rule that matched this message, if applicable.
    matched_content: Optional[:class:`str`]
        The content in the message that matched the keyword, if applicable.

        .. note::

            This requires :attr:`Intents.message_conrent` to not be empty.
    """

    def __init__(self, *, data: ActionExecutionPayload, state: ConnectionState) -> None:
        self.guild_id: int = int(data["guild_id"])
        self.guild: Optional[Guild] = state._get_guild(self.guild_id)
        self.channel_id: Optional[int] = get_as_snowflake(data, "channel_id")
        if self.guild is not None and self.channel_id is not None:
            self.channel: Optional[GuildChannel] = self.guild.get_channel(self.channel_id)
        else:
            self.channel: Optional[GuildChannel] = None
        self.message_id: Optional[int] = get_as_snowflake(data, "message_id")
        self.message: Optional[Message] = state._get_message(self.message_id)
        self.alert_system_message_id: Optional[int] = get_as_snowflake(
            data, "alert_system_message_id"
        )
        self.alert_system_message: Optional[Message] = state._get_message(
            self.alert_system_message_id
        )
        self.action: AutoModerationAction = AutoModerationAction.from_data(data=data["action"])
        self.rule_id: int = int(data["rule_id"])
        self.rule_trigger_type: AutoModerationTriggerType = try_enum(
            AutoModerationTriggerType, data["rule_trigger_type"]
        )
        self.member_id: int = int(data["user_id"])
        self.member: Optional[Member] = (
            self.guild.get_member(self.member_id) if self.guild is not None else None
        )
        self.content: str = data.get("content", "")
        self.matched_keyword: Optional[str] = data["matched_keyword"]
        self.matched_content: Optional[str] = data.get("matched_content", "")



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands.core import (
    MISSING,
    TYPE_CHECKING,
    Any,
    ArgumentParsingError,
    BadArgument,
    BadBoolArgument,
    BadColorArgument,
    BadColourArgument,
    BadFlagArgument,
    BadInviteArgument,
    BadLiteralArgument,
    BadUnionArgument,
    BotMissingAnyRole,
    BotMissingPermissions,
    BotMissingRole,
    BucketType,
    Callable,
    ChannelNotFound,
    ChannelNotReadable,
    CheckAnyFailure,
    CheckFailure,
    Cog,
    CogT,
    Command,
    CommandError,
    CommandInvokeError,
    CommandNotFound,
    CommandOnCooldown,
    CommandRegistrationError,
    CommandT,
    Context,
    ContextT,
    ConversionError,
    Cooldown,
    CooldownMapping,
    Dict,
    DisabledCommand,
    DynamicCooldownMapping,
    EmojiNotFound,
    ErrorT,
    ExpectedClosingQuoteError,
    ExtensionAlreadyLoaded,
    ExtensionError,
    ExtensionFailed,
    ExtensionNotFound,
    ExtensionNotLoaded,
    FlagError,
    Generator,
    Generic,
    Greedy,
    Group,
    GroupMixin,
    GroupT,
    GuildNotFound,
    GuildStickerNotFound,
    HookT,
    InvalidEndOfQuotedStringError,
    List,
    Literal,
    MaxConcurrency,
    MaxConcurrencyReached,
    MemberNotFound,
    MessageNotFound,
    MissingAnyRole,
    MissingFlagArgument,
    MissingPermissions,
    MissingRequiredArgument,
    MissingRequiredFlag,
    MissingRole,
    NoEntryPointError,
    NoPrivateMessage,
    NotOwner,
    NSFWChannelRequired,
    ObjectNotFound,
    Optional,
    P,
    PartialEmojiConversionFailure,
    PrivateMessageOnly,
    RoleNotFound,
    Set,
    T,
    ThreadNotFound,
    TooManyArguments,
    TooManyFlags,
    Tuple,
    Type,
    TypeVar,
    UnexpectedQuoteError,
    Union,
    UserInputError,
    UserNotFound,
    _BaseCommand,
    _CaseInsensitiveDict,
    after_invoke,
    annotations,
    asyncio,
    before_invoke,
    bot_has_any_role,
    bot_has_guild_permissions,
    bot_has_permissions,
    bot_has_role,
    check,
    check_any,
    command,
    cooldown,
    datetime,
    dm_only,
    dynamic_cooldown,
    functools,
    get_converter,
    get_signature_parameters,
    group,
    guild_only,
    has_any_role,
    has_guild_permissions,
    has_permissions,
    has_role,
    hooked_wrapped_callback,
    inspect,
    is_nsfw,
    is_owner,
    max_concurrency,
    overload,
    run_converters,
    unwrap_function,
    versacord,
    wrap_callback,
)

__all__ = (
    "Command",
    "Group",
    "GroupMixin",
    "command",
    "group",
    "has_role",
    "has_permissions",
    "has_any_role",
    "check",
    "check_any",
    "before_invoke",
    "after_invoke",
    "bot_has_role",
    "bot_has_permissions",
    "bot_has_any_role",
    "cooldown",
    "dynamic_cooldown",
    "max_concurrency",
    "dm_only",
    "guild_only",
    "is_owner",
    "is_nsfw",
    "has_guild_permissions",
    "bot_has_guild_permissions",
)



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.member import (
    MISSING,
    TYPE_CHECKING,
    ActivityTypes,
    Any,
    Asset,
    BaseUser,
    Colour,
    Dict,
    List,
    Literal,
    M,
    Member,
    Object,
    Optional,
    Permissions,
    Status,
    Tuple,
    Type,
    TypeVar,
    Union,
    User,
    VoiceState,
    _UserTag,
    abc,
    annotations,
    attrgetter,
    create_activity,
    datetime,
    flatten_user,
    inspect,
    itertools,
    overload,
    sys,
    try_enum,
    utils,
)

__all__ = ("VoiceState", "Member")



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.errors import (
    TYPE_CHECKING,
    Any,
    ClientException,
    ConnectionClosed,
    Dict,
    DiscordException,
    DiscordServerError,
    Forbidden,
    GatewayNotFound,
    HTTPException,
    InteractionResponded,
    InvalidArgument,
    InvalidData,
    List,
    LoginFailure,
    NoMoreItems,
    NotFound,
    Optional,
    PrivilegedIntentsRequired,
    Tuple,
    Union,
    _flatten_error_dict,
    annotations,
)

__all__ = (
    "DiscordException",
    "ClientException",
    "NoMoreItems",
    "GatewayNotFound",
    "HTTPException",
    "Forbidden",
    "NotFound",
    "DiscordServerError",
    "InvalidData",
    "InvalidArgument",
    "LoginFailure",
    "ConnectionClosed",
    "PrivilegedIntentsRequired",
    "InteractionResponded",
)



# SPDX-License-Identifier: MIT

from __future__ import annotations

from typing import List, Literal, Optional, TypedDict

from typing_extensions import NotRequired

from .snowflake import Snowflake
from .user import PartialUser

StatusType = Literal["idle", "dnd", "online", "offline"]


class PartialPresenceUpdate(TypedDict):
    user: PartialUser
    guild_id: Snowflake
    status: StatusType
    activities: List[Activity]
    client_status: ClientStatus


class ClientStatus(TypedDict, total=False):
    desktop: str
    mobile: str
    web: str


class ActivityTimestamps(TypedDict, total=False):
    start: int
    end: int


class ActivityParty(TypedDict, total=False):
    id: str
    size: List[int]


class ActivityAssets(TypedDict, total=False):
    large_image: str
    large_text: str
    small_image: str
    small_text: str


class ActivitySecrets(TypedDict, total=False):
    join: str
    spectate: str
    match: str


class ActivityEmoji(TypedDict):
    id: NotRequired[Snowflake]
    animated: NotRequired[bool]
    name: str


class ActivityButton(TypedDict):
    label: str
    url: str


ActivityType = Literal[0, 1, 2, 4, 5]


class SendableActivity(TypedDict):
    name: str
    type: ActivityType
    url: NotRequired[Optional[str]]


class _BaseActivity(SendableActivity):
    created_at: int


class Activity(_BaseActivity, total=False):
    state: Optional[str]
    details: Optional[str]
    timestamps: ActivityTimestamps
    assets: ActivityAssets
    party: ActivityParty
    application_id: Snowflake
    flags: int
    emoji: Optional[ActivityEmoji]
    secrets: ActivitySecrets
    session_id: Optional[str]
    instance: bool
    buttons: List[ActivityButton]



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands.flags import (
    MISSING,
    TYPE_CHECKING,
    Any,
    BadFlagArgument,
    CommandError,
    Dict,
    F,
    Flag,
    FlagConverter,
    FlagsMeta,
    Iterator,
    List,
    Literal,
    MissingFlagArgument,
    MissingRequiredFlag,
    Optional,
    Pattern,
    Set,
    StringView,
    TooManyFlags,
    Tuple,
    Type,
    TypeVar,
    Union,
    annotations,
    convert_flag,
    dataclass,
    field,
    flag,
    get_flags,
    inspect,
    maybe_coroutine,
    re,
    resolve_annotation,
    run_converters,
    sys,
    tuple_convert_all,
    tuple_convert_flag,
    validate_flag_name,
)

__all__ = ("Flag", "flag", "FlagConverter")



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.invite import (
    GatewayInvite,
    GatewayInviteCreate,
    GatewayInviteDelete,
    IncompleteInvite,
    Invite,
    InviteGuild,
    InviteTargetType,
    InviteWithCounts,
    Literal,
    Optional,
    PartialAppInfo,
    PartialChannel,
    PartialUser,
    Snowflake,
    TypedDict,
    Union,
    VanityInvite,
    _GatewayInviteCreateOptional,
    _GatewayInviteDeleteOptional,
    _GuildPreviewUnique,
    _InviteMetadata,
    _InviteOptional,
    annotations,
)



# SPDX-License-Identifier: MIT

from __future__ import annotations

import asyncio
from typing import TYPE_CHECKING, Callable, Generic, List, Optional, Tuple, TypeVar, Union

from ...components import SelectOption, StringSelectMenu
from ...emoji import Emoji
from ...enums import ComponentType
from ...interactions import ClientT
from ...partial_emoji import PartialEmoji
from ...utils import MISSING
from ..item import ItemCallbackType
from ..view import View
from .base import SelectBase

if TYPE_CHECKING:
    from typing_extensions import Self

__all__ = (
    "Select",
    "select",
    "StringSelect",
    "string_select",
)

V = TypeVar("V", bound="View", covariant=True)


class StringSelect(SelectBase, Generic[V]):
    """Represents a UI string select menu.

    This is usually represented as a drop down menu.

    In order to get the selected items that the user has chosen, use :attr:`StringSelect.values`.

    There is an alias for this class called ``Select``.

    .. versionadded:: 2.0

    Parameters
    ----------
    custom_id: :class:`str`
        The ID of the select menu that gets received during an interaction.
        If not given then one is generated for you.
    placeholder: Optional[:class:`str`]
        The placeholder text that is shown if nothing is selected, if any.
    min_values: :class:`int`
        The minimum number of items that must be chosen for this select menu.
        Defaults to 1 and must be between 1 and 25.
    max_values: :class:`int`
        The maximum number of items that must be chosen for this select menu.
        Defaults to 1 and must be between 1 and 25.
    options: List[:class:`.SelectOption`]
        A list of options that can be selected in this menu.
    disabled: :class:`bool`
        Whether the select is disabled or not.
    row: Optional[:class:`int`]
        The relative row this select menu belongs to. A Discord component can only have 5
        rows. By default, items are arranged automatically into those 5 rows. If you'd
        like to control the relative positioning of the row then passing an index is advised.
        For example, row=1 will show up before row=2. Defaults to ``None``, which is automatic
        ordering. The row number must be between 0 and 4 (i.e. zero indexed).
    """

    __item_repr_attributes__: Tuple[str, ...] = (
        "placeholder",
        "min_values",
        "max_values",
        "options",
        "disabled",
    )

    def __init__(
        self,
        *,
        custom_id: str = MISSING,
        placeholder: Optional[str] = None,
        min_values: int = 1,
        max_values: int = 1,
        options: List[SelectOption] = MISSING,
        disabled: bool = False,
        row: Optional[int] = None,
    ) -> None:
        super().__init__(
            custom_id=custom_id,
            min_values=min_values,
            max_values=max_values,
            disabled=disabled,
            row=row,
            placeholder=placeholder,
        )
        self._selected_values: List[str] = []
        options = [] if options is MISSING else options
        self._underlying = StringSelectMenu._raw_construct(
            custom_id=self.custom_id,
            type=ComponentType.select,
            placeholder=self.placeholder,
            min_values=self.min_values,
            max_values=self.max_values,
            disabled=self.disabled,
            options=options,
        )

    @property
    def options(self) -> List[SelectOption]:
        """List[:class:`.SelectOption`]: A list of options that can be selected in this menu."""
        return self._underlying.options

    @options.setter
    def options(self, value: List[SelectOption]) -> None:
        if not isinstance(value, list):
            raise TypeError("options must be a list of SelectOption")
        if not all(isinstance(obj, SelectOption) for obj in value):
            raise TypeError("All list items must subclass SelectOption")

        self._underlying.options = value

    def add_option(
        self,
        *,
        label: str,
        value: str = MISSING,
        description: Optional[str] = None,
        emoji: Optional[Union[str, Emoji, PartialEmoji]] = None,
        default: bool = False,
    ) -> None:
        """Adds an option to the select menu.

        To append a pre-existing :class:`.SelectOption` use the
        :meth:`append_option` method instead.

        Parameters
        ----------
        label: :class:`str`
            The label of the option. This is displayed to users.
            Can only be up to 100 characters.
        value: :class:`str`
            The value of the option. This is not displayed to users.
            If not given, defaults to the label. Can only be up to 100 characters.
        description: Optional[:class:`str`]
            An additional description of the option, if any.
            Can only be up to 100 characters.
        emoji: Optional[Union[:class:`str`, :class:`.Emoji`, :class:`.PartialEmoji`]]
            The emoji of the option, if available. This can either be a string representing
            the custom or unicode emoji or an instance of :class:`.PartialEmoji` or :class:`.Emoji`.
        default: :class:`bool`
            Whether this option is selected by default.

        Raises
        ------
        ValueError
            The number of options exceeds 25.
        """

        option = SelectOption(
            label=label,
            value=value,
            description=description,
            emoji=emoji,
            default=default,
        )

        self.append_option(option)

    def append_option(self, option: SelectOption) -> None:
        """Appends an option to the select menu.

        Parameters
        ----------
        option: :class:`.SelectOption`
            The option to append to the select menu.

        Raises
        ------
        ValueError
            The number of options exceeds 25.
        """

        if len(self._underlying.options) > 25:
            raise ValueError("maximum number of options already provided")

        self._underlying.options.append(option)

    @classmethod
    def from_component(cls, component: StringSelectMenu) -> Self:
        return cls(
            custom_id=component.custom_id,
            placeholder=component.placeholder,
            min_values=component.min_values,
            max_values=component.max_values,
            options=component.options,
            disabled=component.disabled,
            row=None,
        )


def string_select(
    *,
    placeholder: Optional[str] = None,
    custom_id: str = MISSING,
    min_values: int = 1,
    max_values: int = 1,
    options: List[SelectOption] = MISSING,
    disabled: bool = False,
    row: Optional[int] = None,
) -> Callable[
    [ItemCallbackType[StringSelect[V], ClientT]], ItemCallbackType[StringSelect[V], ClientT]
]:
    """A decorator that attaches a string select menu to a component.

    There is an alias for this function called ``select``.

    The function being decorated should have three parameters, ``self`` representing
    the :class:`.ui.View`, the :class:`.ui.StringSelect` being pressed and
    the :class:`.Interaction` you receive.

    In order to get the selected items that the user has chosen within the callback
    use :attr:`StringSelect.values`.

    Parameters
    ----------
    placeholder: Optional[:class:`str`]
        The placeholder text that is shown if nothing is selected, if any.
    custom_id: :class:`str`
        The ID of the select menu that gets received during an interaction.
        It is recommended not to set this parameter to prevent conflicts.
    row: Optional[:class:`int`]
        The relative row this select menu belongs to. A Discord component can only have 5
        rows. By default, items are arranged automatically into those 5 rows. If you'd
        like to control the relative positioning of the row then passing an index is advised.
        For example, row=1 will show up before row=2. Defaults to ``None``, which is automatic
        ordering. The row number must be between 0 and 4 (i.e. zero indexed).
    min_values: :class:`int`
        The minimum number of items that must be chosen for this select menu.
        Defaults to 1 and must be between 1 and 25.
    max_values: :class:`int`
        The maximum number of items that must be chosen for this select menu.
        Defaults to 1 and must be between 1 and 25.
    options: List[:class:`.SelectOption`]
        A list of options that can be selected in this menu.
    disabled: :class:`bool`
        Whether the select is disabled or not. Defaults to ``False``.
    """

    def decorator(func: ItemCallbackType[Select[V]]) -> ItemCallbackType[Select[V]]:
        if not asyncio.iscoroutinefunction(func):
            raise TypeError("Select function must be a coroutine function")

        func.__discord_ui_model_type__ = StringSelect
        func.__discord_ui_model_kwargs__ = {
            "placeholder": placeholder,
            "custom_id": custom_id,
            "row": row,
            "min_values": min_values,
            "max_values": max_values,
            "options": options,
            "disabled": disabled,
        }
        return func

    return decorator


Select = StringSelect
select = string_select
"""alias of :func:`string_select`"""



# SPDX-License-Identifier: MIT

from __future__ import annotations

from typing import Literal, Optional, TypedDict

from typing_extensions import NotRequired

from .channel import PartialChannel
from .snowflake import Snowflake
from .user import User


class SourceGuild(TypedDict):
    id: int
    name: str
    icon: str


WebhookType = Literal[1, 2, 3]


class _FollowerWebhookOptional(TypedDict, total=False):
    source_channel: PartialChannel
    source_guild: SourceGuild


class FollowerWebhook(_FollowerWebhookOptional):
    channel_id: Snowflake
    webhook_id: Snowflake


class PartialWebhook(TypedDict):
    id: Snowflake
    type: WebhookType
    guild_id: NotRequired[Snowflake]
    user: NotRequired[User]
    token: NotRequired[str]


class _FullWebhook(TypedDict, total=False):
    name: Optional[str]
    avatar: Optional[str]
    channel_id: Snowflake
    application_id: Optional[Snowflake]


class Webhook(PartialWebhook, _FullWebhook):
    ...



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.oggparse import (
    IO,
    TYPE_CHECKING,
    ClassVar,
    DiscordException,
    Generator,
    OggError,
    OggPage,
    OggStream,
    Optional,
    Tuple,
    annotations,
    struct,
)

__all__ = ("OggError", "OggPage", "OggStream")



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.channel import (
    CategoryChannel,
    Channel,
    ChannelType,
    DMChannel,
    GroupDMChannel,
    GuildChannel,
    List,
    Literal,
    NewsChannel,
    Optional,
    OverwriteType,
    PartialChannel,
    PartialUser,
    PermissionOverwrite,
    PrivacyLevel,
    Snowflake,
    StageChannel,
    StageInstance,
    TextChannel,
    ThreadArchiveDuration,
    ThreadChannel,
    ThreadMember,
    ThreadMetadata,
    TypedDict,
    Union,
    VideoQualityMode,
    VoiceChannel,
    _BaseChannel,
    _BaseGuildChannel,
    _StageChannelOptional,
    _TextChannelOptional,
    _ThreadChannelOptional,
    _VoiceChannelOptional,
)



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.webhook import (
    FollowerWebhook,
    Literal,
    Optional,
    PartialChannel,
    PartialWebhook,
    Snowflake,
    SourceGuild,
    TypedDict,
    User,
    Webhook,
    WebhookType,
    _FollowerWebhookOptional,
    _FullWebhook,
    _WebhookOptional,
    annotations,
)



# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands.errors import (
    TYPE_CHECKING,
    Any,
    ArgumentParsingError,
    BadArgument,
    BadBoolArgument,
    BadColorArgument,
    BadColourArgument,
    BadFlagArgument,
    BadInviteArgument,
    BadLiteralArgument,
    BadUnionArgument,
    BotMissingAnyRole,
    BotMissingPermissions,
    BotMissingRole,
    Callable,
    ChannelNotFound,
    ChannelNotReadable,
    CheckAnyFailure,
    CheckFailure,
    ClientException,
    CommandError,
    CommandInvokeError,
    CommandNotFound,
    CommandOnCooldown,
    CommandRegistrationError,
    ConversionError,
    DisabledCommand,
    DiscordException,
    EmojiNotFound,
    ExpectedClosingQuoteError,
    ExtensionAlreadyLoaded,
    ExtensionError,
    ExtensionFailed,
    ExtensionNotFound,
    ExtensionNotLoaded,
    FlagError,
    GuildNotFound,
    GuildStickerNotFound,
    InvalidEndOfQuotedStringError,
    List,
    MaxConcurrencyReached,
    MemberNotFound,
    MessageNotFound,
    MissingAnyRole,
    MissingFlagArgument,
    MissingPermissions,
    MissingRequiredArgument,
    MissingRequiredFlag,
    MissingRole,
    NoEntryPointError,
    NoPrivateMessage,
    NotOwner,
    NSFWChannelRequired,
    ObjectNotFound,
    Optional,
    PartialEmojiConversionFailure,
    PrivateMessageOnly,
    RoleNotFound,
    ThreadNotFound,
    TooManyArguments,
    TooManyFlags,
    Tuple,
    Type,
    UnexpectedQuoteError,
    Union,
    UserInputError,
    UserNotFound,
    annotations,
)

__all__ = (
    "CommandError",
    "MissingRequiredArgument",
    "BadArgument",
    "PrivateMessageOnly",
    "NoPrivateMessage",
    "CheckFailure",
    "CheckAnyFailure",
    "CommandNotFound",
    "DisabledCommand",
    "CommandInvokeError",
    "TooManyArguments",
    "UserInputError",
    "CommandOnCooldown",
    "MaxConcurrencyReached",
    "NotOwner",
    "MessageNotFound",
    "ObjectNotFound",
    "MemberNotFound",
    "GuildNotFound",
    "UserNotFound",
    "ChannelNotFound",
    "ThreadNotFound",
    "ChannelNotReadable",
    "BadColourArgument",
    "BadColorArgument",
    "RoleNotFound",
    "BadInviteArgument",
    "EmojiNotFound",
    "GuildStickerNotFound",
    "PartialEmojiConversionFailure",
    "BadBoolArgument",
    "MissingRole",
    "BotMissingRole",
    "MissingAnyRole",
    "BotMissingAnyRole",
    "MissingPermissions",
    "BotMissingPermissions",
    "NSFWChannelRequired",
    "ConversionError",
    "BadUnionArgument",
    "BadLiteralArgument",
    "ArgumentParsingError",
    "UnexpectedQuoteError",
    "InvalidEndOfQuotedStringError",
    "ExpectedClosingQuoteError",
    "ExtensionError",
    "ExtensionAlreadyLoaded",
    "ExtensionNotLoaded",
    "NoEntryPointError",
    "ExtensionFailed",
    "ExtensionNotFound",
    "CommandRegistrationError",
    "FlagError",
    "BadFlagArgument",
    "MissingFlagArgument",
    "TooManyFlags",
    "MissingRequiredFlag",
)
